/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		Field.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	05.08.2001
// letzte nderung:	27.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Die Klasse Field verwaltet das Spielfeld in einer 2d-Matrix von Bytes
//			(unsigned chars)
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "Field.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::Field
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
//			holt Speicher fr die Matrix
/////////////////////////////////////////////////////////////////////////////////////
Field::Field(void)
{
	int iCountX, iCountY;

	m_iLastParticleUsed=0;
	

	srand( (unsigned)time( NULL ) );


	m_pucFieldData=new unsigned char[iFieldSizeX*iFieldSizeY];
	if (m_pucFieldData==NULL)
	{
		XLogging::log("Field::Field > (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//verdammt
	}

	m_pucBackupData=new unsigned char[iFieldSizeX*iFieldSizeY];
	if (m_pucBackupData==NULL)
	{
		XLogging::log("Field::Field > (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//verdammt
	}

	m_pucDisplayedData=new unsigned char[iFieldSizeX*iFieldSizeY];
	if (m_pucBackupData==NULL)
	{
		XLogging::log("Field::Field > (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//verdammt
	}

	for(iCountX=0;iCountX<iFieldSizeX;iCountX++)
		for(iCountY=0;iCountY<iFieldSizeY;iCountY++)
		{
			m_pucBackupData[iCountX+iFieldSizeX*iCountY]=rand()%16;
			
			m_pucFieldData[iCountX+iFieldSizeX*iCountY]=
				m_pucBackupData[iCountX+iFieldSizeX*iCountY];
		}

	for (iCountX=0;iCountX<iNumberOfPlayers;iCountX++)
	{
		m_iStartX[iCountX]=0;
		m_iStartY[iCountX]=0;
		m_iStartDirection[iCountX]=0;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::~Field
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Destruktor, gibt den Speicher wieder zurck
/////////////////////////////////////////////////////////////////////////////////////
Field::~Field()
{
	delete[] m_pucFieldData;
	delete[] m_pucBackupData;
	delete[] m_pucDisplayedData;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::load
/////////////////////////////////////////////////////////////////////////////////////
// Info:	ldt eine Map mit dem vorgegebenen FileNamen
//			lscht ebenfalls alle Partikel
/////////////////////////////////////////////////////////////////////////////////////
bool Field::load(const char* pcFileName)
{
	int iCountX, iCountY;

	for(iCountX=0;iCountX<iNrOfParticles;iCountX++)
	{
		m_Particles[iCountX].create(0,0,0,0,0,0,0);
	}
	m_iLastParticleUsed=0;

	XLogging::log("Field::load > Versuche File '");
	XLogging::log((char*)pcFileName);
	XLogging::log("' zu laden.\n");

	//File im Binrmodus ffnen
	ifstream isFile(pcFileName,ios::in|ios::binary|ios::nocreate);

	if (isFile.fail())
		return false;

	isFile.read(m_pucBackupData,iFieldSizeX*iFieldSizeY);

	for(iCountX=0;iCountX<iFieldSizeX;iCountX++)
		for(iCountY=0;iCountY<iFieldSizeY;iCountY++)
		{
			m_pucFieldData[iCountX+iFieldSizeX*iCountY]=
			m_pucBackupData[iCountX+iFieldSizeX*iCountY];
		}

	for (iCountX=0;iCountX<iNumberOfPlayers;iCountX++)
	{
		isFile.read((unsigned char*)&m_iStartX[iCountX],sizeof(int));
		isFile.read((unsigned char*)&m_iStartY[iCountX],sizeof(int));
		isFile.read((unsigned char*)&m_iStartDirection[iCountX],sizeof(int));
	}

	if (isFile.eof())
		return false;

	isFile.close();

	XLogging::log("Field::load > Erfolgreich!\n");

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::save
/////////////////////////////////////////////////////////////////////////////////////
// Info:	speichert eine Map mit dem vorgegebenen FileNamen
/////////////////////////////////////////////////////////////////////////////////////
bool Field::save(const char* pcFileName)
{
	int iCountX;

	XLogging::log("Field::save > Versuche File '");
	XLogging::log((char*)pcFileName);
	XLogging::log("' zu speichern.\n");

	//File im Binrmodus ffnen
	ofstream osFile(pcFileName,ios::binary|ios::out);

	if (osFile.fail())
		return false;

	osFile.write(m_pucBackupData,iFieldSizeX*iFieldSizeY);


	for (iCountX=0;iCountX<iNumberOfPlayers;iCountX++)
	{
		osFile.write((unsigned char*)&m_iStartX[iCountX],sizeof(int));
		osFile.write((unsigned char*)&m_iStartY[iCountX],sizeof(int));
		osFile.write((unsigned char*)&m_iStartDirection[iCountX],sizeof(int));
	}

	osFile.close();

	XLogging::log("Field::save > Erfolgreich!\n");

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::render
/////////////////////////////////////////////////////////////////////////////////////
// Info:	rendert das SpielFeld an die Stelle x,y, die Tiles haben die Grsse
//			iSize, wenn DisplayEmpty true is, wird alles gezeichnet, sonst nur das 
//			vernderte
/////////////////////////////////////////////////////////////////////////////////////
void Field::render(	XSurface* pxdDest, 
					XSprite* pxsTileSet, 
					int iX, 
					int iY, 
					int iSize, 
					bool bDisplayEmpty/* =true */)
{
	int iCountX;
	int iCountY;
	int iIndex;
	int iYTmp=iY;

	if (bDisplayEmpty)
	{
		for(iCountX=0;iCountX<iFieldSizeX;iCountX++,iX+=iSize)
		for(iCountY=0,iY=iYTmp;iCountY<iFieldSizeY;iCountY++,iY+=iSize)
		{
			iIndex=iCountX+iFieldSizeX*iCountY;
			pxsTileSet->m_lX=iX;
			pxsTileSet->m_lY=iY;
			pxsTileSet->m_lFrame=m_pucBackupData[iIndex];
			pxsTileSet->blit(pxdDest,DDBLT_WAIT);

			if (m_pucBackupData[iIndex]!=m_pucFieldData[iIndex])
			{
				pxsTileSet->m_lFrame=m_pucFieldData[iIndex];
				pxsTileSet->blit(pxdDest);
			}
			m_pucDisplayedData[iIndex]=m_pucFieldData[iIndex];
		}
	}
	else //if (!bDisplayEmpty)
	{
		for(iCountX=0;iCountX<iFieldSizeX;iCountX++,iX+=iSize)
		for(iCountY=0,iY=iYTmp;iCountY<iFieldSizeY;iCountY++,iY+=iSize)
		{
			iIndex=iCountX+iFieldSizeX*iCountY;
			if (m_pucDisplayedData[iIndex]!=m_pucFieldData[iIndex])
			{
				pxsTileSet->m_lX=iX;
				pxsTileSet->m_lY=iY;
				pxsTileSet->m_lFrame=m_pucBackupData[iIndex];
				pxsTileSet->blit(pxdDest,DDBLT_WAIT);
				
				if (m_pucBackupData[iIndex]!=m_pucFieldData[iIndex])
				{
					pxsTileSet->m_lFrame=m_pucFieldData[iIndex];
					pxsTileSet->blit(pxdDest);
				}
				m_pucDisplayedData[iIndex]=m_pucFieldData[iIndex];
			}
		}
	}

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::renderParticles
/////////////////////////////////////////////////////////////////////////////////////
// Info:	rendert die Partikel (un ihre Schatten)
/////////////////////////////////////////////////////////////////////////////////////
void Field::renderParticles(	XSurface* pxdDest,
								XSprite* pxsParticle,
								int iTime,
								int iX, 
								int iY,
								int iSize)
{
	int iCount;

	for(iCount=0;iCount<iNrOfParticles;iCount++)
	{
		m_Particles[iCount].update(iTime);
		m_Particles[iCount].renderShadow(pxdDest,pxsParticle,iX,iY,iSize);
	}

	for(iCount=0;iCount<iNrOfParticles;iCount++)
	{
		m_Particles[iCount].render(pxdDest,pxsParticle,iX,iY,iSize);
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::createParticle
/////////////////////////////////////////////////////////////////////////////////////
// Info:	erzeugt einen Partikel an entsprechender Stelle und Typ
/////////////////////////////////////////////////////////////////////////////////////
void Field::createParticle(float	fPosX, 
						   float	fPosY, 
						   float	fSpeedX, 
						   float	fSpeedY, 
						   int		iType)
{
	m_iLastParticleUsed++;
	if (m_iLastParticleUsed>=iNrOfParticles)
		m_iLastParticleUsed=0;

	m_Particles[m_iLastParticleUsed].create(
		fPosX,
		fPosY,
		0.5f,
		0.0000025f*(500-rand()%1000)+fSpeedX*0.002f,
		0.0000025f*(500-rand()%1000)+fSpeedY*0.002f,
		0.00003f+0.000005f*(rand()%1000),
		iType);
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::freeTile
/////////////////////////////////////////////////////////////////////////////////////
// Info:	macht das Tile frei, indem das entsprechende Tile aus der BackupMap
//			geholt wird
/////////////////////////////////////////////////////////////////////////////////////
bool Field::freeTile(int iX, int iY)
{
	//liegt es im Feld?
	if (iX<0 || iY<0 || iX>=iFieldSizeX || iY>=iFieldSizeY)
		return false;

	m_pucFieldData[iX+iFieldSizeX*iY]=m_pucBackupData[iX+iFieldSizeX*iY];

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::isTotalEmpty
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt true zurck, wenn das Feld und seine acht Nachbarn leer sind
/////////////////////////////////////////////////////////////////////////////////////
bool Field::isTotalEmpty(int iX, int iY)
{
	int iCountX,iCountY;

	for (iCountX=-1;iCountX<2;iCountX++)
	for (iCountY=-1;iCountY<2;iCountY++)
	{
		if (m_pucFieldData	[(iX+iFieldSizeX+iCountX)%iFieldSizeX
							+iFieldSizeX*((iY+iFieldSizeY+iCountY)%iFieldSizeY)]
			> g_TileSetData.m_iLastEmpty)
			return false;
	}

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::setTile
/////////////////////////////////////////////////////////////////////////////////////
// Info:	setzt eine Tile im Feld, wenn der DevMode auf true ist, wird auch die
//			eigentliche Map verndert (Mapeditor!)
/////////////////////////////////////////////////////////////////////////////////////
bool Field::setTile(int iX, int iY, int iNewTile, bool bDevMode/* =false */)
{
	//liegt es im Feld?
	if (iX<0 || iY<0 || iX>=iFieldSizeX || iY>=iFieldSizeY)
		return false;

	//Tile setzen
	m_pucFieldData[iX+iFieldSizeX*iY]=(unsigned char)iNewTile;

	//bei DevMode Tile auch in der echten Map setzen
	if (bDevMode)
		m_pucBackupData[iX+iFieldSizeX*iY]=(unsigned char)iNewTile;

	return true;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::getTile
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt das Tile an der Stelle iX,iY; gibt -1 bei Fehler zurck
/////////////////////////////////////////////////////////////////////////////////////
int Field::getTile(int iX, int iY)
{
	//liegt es im Feld?
	if (iX<0 || iY<0 || iX>=iFieldSizeX || iY>=iFieldSizeY)
		return -1;
	
	return (int)m_pucFieldData[iX+iFieldSizeX*iY];
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::getStartX
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt die X-Koordinate des Startpunktes des Players iNr, wenn der
//			zweite Parameter benutzt wird, ist ein Setzen des Startpunktes mglich
/////////////////////////////////////////////////////////////////////////////////////
int Field::getStartX(int iNr, int iSetX/* =-1 */)
{
	//gibts den Spieler berhaupt?
	if (iNr<0 || iNr>=iNumberOfPlayers)
		return -1;

	//soll etwas gesetzt werden?
	if (iSetX>=0 && iSetX<iFieldSizeX)
		m_iStartX[iNr]=iSetX;

	return m_iStartX[iNr];
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::getStartY
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt die Y-Koordinate des Startpunktes des Players iNr, wenn der
//			zweite Parameter benutzt wird, ist ein Setzen des Startpunktes mglich
/////////////////////////////////////////////////////////////////////////////////////
int Field::getStartY(int iNr, int iSetY/* =-1 */)
{
	//gibts den Spieler berhaupt?
	if (iNr<0 || iNr>=iNumberOfPlayers)
		return -1;

	//soll etwas gesetzt werden?
	if (iSetY>=0 && iSetY<iFieldSizeY)
		m_iStartY[iNr]=iSetY;

	return m_iStartY[iNr];
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Field::getStartDirection
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt die Startrichtung des Players iNr, wenn der zweite Parameter
//			benutzt wird, ist ein Setzen der Startrichtung mglich
/////////////////////////////////////////////////////////////////////////////////////
int Field::getStartDirection(int iNr, int iSetDirection/* =-1 */)
{
	//gibts den Spieler berhaupt?
	if (iNr<0 || iNr>=iNumberOfPlayers)
		return -1;

	//soll etwas gesetzt werden?
	if (iSetDirection>=0 && iSetDirection<4)
		m_iStartDirection[iNr]=iSetDirection;

	return m_iStartDirection[iNr];
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	Field.cpp
/////////////////////////////////////////////////////////////////////////////////////
