/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		PPIntro.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	24.07.2001
// letzte nderung:	21.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Das TNG-Soft Intro
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "PPIntro.h"

// Methoden-Definitionen


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPIntro::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Grafiken Laden, Timer zurcksetzen
//
/////////////////////////////////////////////////////////////////////////////////////
HRESULT PPIntro::initialize(XDisplay* pxdisplayDest)
{
	HRESULT hres;

	XLogging::log(" --- Intro ---\n");

	hres=m_xsurfBackground.initialize(pxdisplayDest,"logo\\backgr.bmp");

	hres=m_xsurfSnakeLogo.initialize(pxdisplayDest,"gfx\\snlogo.bmp");

	hres=m_xsTNG.initialize(pxdisplayDest,"logo\\tng.spr");
	m_xsTNG.m_bVisible=true;

	hres=m_xsSOFT.initialize(pxdisplayDest,"logo\\soft.spr");
	m_xsSOFT.m_bVisible=true;

	hres=m_xsStarFX.initialize(pxdisplayDest,"logo\\stars.spr");
	m_xsStarFX.m_bVisible=false;

	m_pfsSound1=g_xfmodHelper.loadSample("audio\\intro1.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_2D);
	m_pfsSound2=g_xfmodHelper.loadSample("audio\\intro2.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_2D);

	m_iTimePassed=0;
	m_bFinish=false;

	return hres;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPIntro::unitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Grafiken/Sound kicken
/////////////////////////////////////////////////////////////////////////////////////
HRESULT PPIntro::uninitialize(void)
{
	HRESULT hres;

	FSOUND_StopSound(FSOUND_ALL);
	FSOUND_Sample_Free(m_pfsSound1);
	FSOUND_Sample_Free(m_pfsSound2);

	hres=m_xsurfBackground.uninitialize();

	hres=m_xsurfSnakeLogo.uninitialize();

	hres=m_xsTNG.uninitialize();

	hres=m_xsSOFT.uninitialize();

	hres=m_xsStarFX.uninitialize();

	return hres;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPIntro::recoverFromFocusLost
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt Surface zurueck
/////////////////////////////////////////////////////////////////////////////////////
HRESULT PPIntro::recoverFromFocusLost(void)
{
	HRESULT hres;

	hres=m_xsurfBackground.restore();

	hres=m_xsurfSnakeLogo.restore();

	hres=m_xsTNG.restore();

	hres=m_xsSOFT.restore();

	hres=m_xsStarFX.restore();

	return hres;	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPIntro::run
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Eigentlicher Ablauf des Intros
/////////////////////////////////////////////////////////////////////////////////////
State PPIntro::run(int iTime, XDisplay* pxdisplayDest)
{
	m_iTimePassed+=iTime;
	
	if (m_iTimePassed > 8000)
	{
		m_xsurfSnakeLogo.blit(pxdisplayDest);
	}
	else
	{
		m_xsurfBackground.blit(pxdisplayDest);
		
		if (m_iTimePassed < 4000)
		{
			DWORD dwS2k=4000-m_iTimePassed;

			m_xsTNG.m_lX=116;
			m_xsTNG.m_lY=	(long)(100-(dwS2k*dwS2k)*0.00015);
			m_xsSOFT.m_lX=(long)(162+(dwS2k*dwS2k)*0.0002);
			m_xsSOFT.m_lY=300;
		}
		else
		{
			m_xsTNG.m_lX=116;
			m_xsTNG.m_lY=100;
			m_xsSOFT.m_lX=162;
			m_xsSOFT.m_lY=300;
		}
		
		if (m_iTimePassed>1000 && m_iTimePassed-iTime<=1000)
		{
			g_xfmodHelper.playSample(m_pfsSound1);
		}
		
		m_xsStarFX.m_bVisible=false;
		
		if (m_iTimePassed > 4100 && m_iTimePassed < 4600)
		{
			m_xsStarFX.m_bVisible=true;
			m_xsStarFX.m_lX=680;
			m_xsStarFX.m_lY=307;
			m_xsStarFX.m_lFrame=(long)(7.9-abs(4350-m_iTimePassed)*0.0316);
			
			if (m_iTimePassed-iTime<=4100)
			{
				g_xfmodHelper.playSample(m_pfsSound2,208);
			}
		}
		if (m_iTimePassed > 4700 && m_iTimePassed < 5200)
		{
			m_xsStarFX.m_bVisible=true;
			m_xsStarFX.m_lX=187;
			m_xsStarFX.m_lY=712;
			m_xsStarFX.m_lFrame=(long)(7.9-abs(4950-m_iTimePassed)*0.0316);
			
			if (m_iTimePassed-iTime<=4700)
		{
			g_xfmodHelper.playSample(m_pfsSound2,16);
		}
		}
		if (m_iTimePassed > 5300 && m_iTimePassed < 5800)
		{
			m_xsStarFX.m_bVisible=true;
			m_xsStarFX.m_lX=890;
			m_xsStarFX.m_lY=403;
			m_xsStarFX.m_lFrame=(long)(7.9-abs(5550-m_iTimePassed)*0.0316);

			if (m_iTimePassed-iTime<=5300)
			{
				g_xfmodHelper.playSample(m_pfsSound2,240);
			}
		}
		
		m_xsTNG.blit(pxdisplayDest);
		m_xsSOFT.blit(pxdisplayDest);
		m_xsStarFX.blit(pxdisplayDest);
	}

	if (m_bFinish && m_iTimePassed< 8000)
	{
		m_bFinish=false;
		m_iTimePassed=8000;
	}

	if (m_iTimePassed> 16000 || m_bFinish)
	{
		return Menu;
	}
	
	return Intro;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPIntro::wndProc
/////////////////////////////////////////////////////////////////////////////////////
// Info:	wndProc Teil des Intro, ziemlich leer, ghn...
/////////////////////////////////////////////////////////////////////////////////////
void PPIntro::wndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_KEYDOWN:
			switch (wParam)
			{
				case VK_ESCAPE: 
					m_bFinish=true;
					break;
			}	
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	PPIntro.cpp
/////////////////////////////////////////////////////////////////////////////////////
