/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		PPMenu.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	01.08.2001
// letzte nderung:	07.11.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Menu-Implemetationen			
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "PPMenu.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Grafiken Laden
//
/////////////////////////////////////////////////////////////////////////////////////
HRESULT PPMenu::initialize(XDisplay* pxdisplayDest)
{
	HRESULT hres;

	XLogging::log(" --- Menu ---\n");

	m_iChoosen=0;
	m_iChoosenState=0;
	m_iChoosenTime=0;

	//ignore a pressed RETURN key
	m_iKeyPressed=DIK_RETURN;

	hres=m_xsurfBackground.initialize(pxdisplayDest,"gfx\\menuback.bmp");

	hres=m_xsPanels.initialize(pxdisplayDest,"gfx\\menupanels.spr");
	m_xsPanels.m_bVisible=true;

	hres=m_xsScreenHolder.initialize(pxdisplayDest,"gfx\\menuadd1.spr");
	m_xsScreenHolder.m_bVisible=true;

	hres=m_xsScreenBack.initialize(pxdisplayDest,"gfx\\menuadd2.spr");
	m_xsScreenBack.m_bVisible=true;

	hres=m_xsMoveIcons.initialize(pxdisplayDest,"gfx\\menuadd3.spr");
	m_xsMoveIcons.m_bVisible=true;

	hres=m_xsSmallTileSet.initialize(pxdisplayDest,"gfx\\TileSetSmall.spr");
	m_xsSmallTileSet.m_bVisible=true;

	hres=m_xsurfMapPreview.initialize(pxdisplayDest,144,144);

	hres=m_xsSpeedBar.initialize(pxdisplayDest,"gfx\\speed1.spr");
	m_xsSpeedBar.m_bVisible=true;

	hres=m_xsSpeedButton.initialize(pxdisplayDest,"gfx\\speed2.spr");
	m_xsSpeedButton.m_bVisible=true;

	hres=m_xfChars.initialize(pxdisplayDest,"gfx\\charset.fnt");

	g_configData.loadLevel(&m_fieldPreview);

	m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);


	m_pfsUp=g_xfmodHelper.loadSample(	"audio\\Menu_up.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_HW3D );

	m_pfsDown=g_xfmodHelper.loadSample(	"audio\\Menu_down.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_HW3D );

	m_pfsInc=g_xfmodHelper.loadSample(	"audio\\Menu_inc.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_HW3D );

	m_pfsDec=g_xfmodHelper.loadSample(	"audio\\Menu_dec.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_HW3D );

	m_pfsMisc=g_xfmodHelper.loadSample(	"audio\\Menu_misc.mp3",
										FSOUND_FREE,
										FSOUND_LOOP_OFF | FSOUND_16BITS | FSOUND_HW3D );

	m_pfmMenuMuzak=g_xfmodHelper.loadModule("audio\\menu.it");
	g_xfmodHelper.playModule(m_pfmMenuMuzak);

	return hres;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::unitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Grafiken/Sound kicken
/////////////////////////////////////////////////////////////////////////////////////
HRESULT PPMenu::uninitialize(void)
{
	HRESULT hres;

	FMUSIC_StopSong(m_pfmMenuMuzak);
	FMUSIC_FreeSong(m_pfmMenuMuzak);
	
	FSOUND_StopSound(FSOUND_ALL);

	FSOUND_Sample_Free(m_pfsUp);
	FSOUND_Sample_Free(m_pfsDown);
	FSOUND_Sample_Free(m_pfsInc);
	FSOUND_Sample_Free(m_pfsDec);
	FSOUND_Sample_Free(m_pfsMisc);
	
	hres=m_xsurfBackground.uninitialize();

	hres=m_xsPanels.uninitialize();

	hres=m_xsScreenHolder.uninitialize();

	hres=m_xsScreenBack.uninitialize();

	hres=m_xsMoveIcons.uninitialize();

	hres=m_xsSmallTileSet.uninitialize();

	hres=m_xsurfMapPreview.uninitialize();

	hres=m_xsSpeedBar.uninitialize();

	hres=m_xsSpeedButton.uninitialize();

	hres=m_xfChars.uninitialize();

	return hres;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::recoverFromFocusLost
/////////////////////////////////////////////////////////////////////////////////////
// Info:	holt Surfaces zurueck
/////////////////////////////////////////////////////////////////////////////////////
HRESULT PPMenu::recoverFromFocusLost(void)
{
	HRESULT hres;

	hres=m_xsurfBackground.restore();

	hres=m_xsPanels.restore();

	hres=m_xsScreenHolder.restore();

	hres=m_xsScreenBack.restore();

	hres=m_xsMoveIcons.restore();

	hres=m_xsSmallTileSet.restore();

	hres=m_xsurfMapPreview.restore();

	hres=m_xsSpeedBar.restore();

	hres=m_xsSpeedButton.restore();

	hres=m_xfChars.restore();

	return hres;	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::run
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Eigentlicher Ablauf des Menus
/////////////////////////////////////////////////////////////////////////////////////
State PPMenu::run(int iTime, XDisplay* pxdisplayDest)
{
	m_stateNext=Menu;
	m_iChoosenTime+=iTime;
	
	int iTmp;
	
	//Konstanten fuer die Menuplatzierung
	const int	iNrOfChoices=5;
	const int	iPanelX=25;
	const int	iPanelY=185;
	const int	iPaneldeltaY=90;

	m_xsurfBackground.blit(pxdisplayDest);

	for (iTmp=0;iTmp<iNrOfChoices;iTmp++)
	{
		m_xsPanels.m_lY=iPanelY+iPaneldeltaY*iTmp;
		m_xsPanels.m_lX=iPanelX;
		m_xsPanels.m_lFrame=2*iTmp;
		if (m_iChoosen==iTmp)
		{
			renderSubMenu(pxdisplayDest,iTmp,iPanelX+500,m_xsPanels.m_lY-110);
			m_xsPanels.m_lFrame++;
		}
		m_xsPanels.blit(pxdisplayDest);
	}

	iTmp=processSubMenu(m_iChoosen);

	switch(iTmp)
	{
	case DIK_DOWN:
		m_iChoosen++;
		m_iChoosenTime=0;
		m_iChoosenState=0;
		if (m_iChoosen==iNrOfChoices)
			m_iChoosen=0;
		g_xfmodHelper.playSample(m_pfsDown);
		break;
	case DIK_UP:
		m_iChoosen--;
		m_iChoosenTime=0;
		m_iChoosenState=0;
		if (m_iChoosen==-1)
			m_iChoosen=iNrOfChoices-1;
		g_xfmodHelper.playSample(m_pfsUp);
		break;
	}

	return m_stateNext;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::wndProc
/////////////////////////////////////////////////////////////////////////////////////
// Info:	wndProc Teil des Menu, ziemlich leer, ghn...
/////////////////////////////////////////////////////////////////////////////////////
void PPMenu::wndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::analyzeKeys
/////////////////////////////////////////////////////////////////////////////////////
// Info:	berprft ob gerade eben eine Taste runtergedrckt wurde, liefert die
//			Nummer zurck, oder ne -1
/////////////////////////////////////////////////////////////////////////////////////
int PPMenu::analyzeKeys()
{
	int		iTmp;
	int		iKeyFound=-1;
		
	g_xiInput.getKeyboardData();

	for(iTmp=0;iTmp<256;iTmp++)
	{
		if (g_xiInput.m_keyboardData[iTmp]!=0)
		{
			iKeyFound=iTmp;
		}
	}

	if (m_iKeyPressed==iKeyFound)
	{
		return -1;
	}

	m_iKeyPressed=iKeyFound;
	return iKeyFound;

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::renderSubMenu
/////////////////////////////////////////////////////////////////////////////////////
// Info:	zeichnet das Submenu iNr an die gegebene Position
/////////////////////////////////////////////////////////////////////////////////////
void PPMenu::renderSubMenu(XDisplay* pxdisplayDest, int iNr, int iPosX, int iPosY)
{	
	int iAway=0;	//wie weit bewegen sich die Klammern noch?
	int iCurrentFrame=4; //welcher Frame wird fr den Screen benutzt (negativ= kein Screen)
	int iCountX;
	int iCountY;

	char pcTmp[256];


	if (m_iChoosenTime<500)
	{
		iAway=(500-m_iChoosenTime)*(500-m_iChoosenTime)/4000;
	}

	if (m_iChoosenTime<700)
	{
		iCurrentFrame=(m_iChoosenTime)/50-9;
	}

	m_xsScreenHolder.m_lY=iPosY-5;
	m_xsScreenHolder.m_lX=iPosX-118-iAway;
	m_xsScreenHolder.m_lFrame=0;
	m_xsScreenHolder.blit(pxdisplayDest);

	m_xsScreenHolder.m_lY=iPosY-5;
	m_xsScreenHolder.m_lX=iPosX+398+iAway;
	m_xsScreenHolder.m_lFrame=1;
	m_xsScreenHolder.blit(pxdisplayDest);

	if (iCurrentFrame>=0)
	{
		for (iCountX=0;iCountX<5;iCountX++)
			for (iCountY=0;iCountY<5;iCountY++)
			{
				m_xsScreenBack.m_lY=iPosY+60*iCountY;
				m_xsScreenBack.m_lX=iPosX+80*iCountX;
				m_xsScreenBack.m_lFrame=iCurrentFrame;
				m_xsScreenBack.blit(pxdisplayDest);
			}
	}

	if (m_iChoosenTime>700)
	{
		switch(iNr)
		{
		case 0://MenuPunkt Start
			m_xfChars.textOut(pxdisplayDest,iPosX+88,iPosY+50,"press enter to begin game!");
			sprintf(pcTmp,"< level [%02d] >",g_configData.getLevelNr());
			m_xfChars.textOut(pxdisplayDest,iPosX+137,iPosY+250,pcTmp);
			m_xsurfMapPreview.blit(pxdisplayDest,iPosX+128,iPosY+90);
			break;

		case 1://MenuPunkt HighScores
			m_xfChars.textOut(	pxdisplayDest,iPosX+92,iPosY+70,
				"f1  wrap around  [     ]");
			if (m_iChoosenState%2 == 1)
				m_xfChars.textOut(pxdisplayDest,iPosX+267,iPosY+70,"on");
			else
				m_xfChars.textOut(pxdisplayDest,iPosX+263,iPosY+70,"off");
			
			if(m_iChoosenState<2) //SinglePlay?
			{
				m_xfChars.textOut(	pxdisplayDest,iPosX+155,iPosY+38,
				"singleplay");
			}
			else
			{
				m_xfChars.textOut(	pxdisplayDest,iPosX+159,iPosY+22,
				"challenge");
				sprintf(pcTmp,"< level [%02d] >",m_iChoosenState/2);
				m_xfChars.textOut(	pxdisplayDest,iPosX+137,iPosY+45,pcTmp);
			}

			for (iCountX=0;iCountX<iHighScoreEntries;iCountX++)
			{
				sprintf(pcTmp,"%1d.",iCountX+1);
				m_xfChars.textOut(	pxdisplayDest,iPosX+90,iPosY+125+26*iCountX,pcTmp);
				iCountY=g_hsData.getEntry(m_iChoosenState,iCountX,pcTmp);
				m_xfChars.textOut(	pxdisplayDest,iPosX+115,iPosY+125+26*iCountX,pcTmp);
				sprintf(pcTmp,"%06d",iCountY);
				m_xfChars.textOut(	pxdisplayDest,iPosX+256,iPosY+125+26*iCountX,pcTmp);
			}

			break;

		case 2://MenuPunkt Options
			switch(m_iChoosenState)
			{
			case 0:// Optionen Seite 1
				switch(g_configData.m_gmActual)
				{
				case SinglePlay:
					m_xfChars.textOut(	pxdisplayDest,iPosX+74,iPosY+34,
										"f1  gamemode  [ singleplay ]");
					sprintf(	pcTmp,
								"    number of players  [ %1d ]",
								g_configData.m_iNrOfActivePlayers);
					m_xfChars.textOut(pxdisplayDest,iPosX+74,iPosY+74,pcTmp);
					break;

				case DeathMatch:
					m_xfChars.textOut(	pxdisplayDest,iPosX+74,iPosY+34,
										"f1  gamemode  [ deathmatch ]");
					sprintf(	pcTmp,
								"f2  number of players  [ %1d ]",
								g_configData.m_iNrOfActivePlayers);
					m_xfChars.textOut(pxdisplayDest,iPosX+74,iPosY+74,pcTmp);
					break;

				case Challenge:
					m_xfChars.textOut(	pxdisplayDest,iPosX+74,iPosY+34,
										"f1  gamemode  [            ]");
					m_xfChars.textOut(	pxdisplayDest,iPosX+222,iPosY+34,
										"challenge");
					sprintf(	pcTmp,
								"    number of players  [ %1d ]",
								g_configData.m_iNrOfActivePlayers);
					m_xfChars.textOut(pxdisplayDest,iPosX+74,iPosY+74,pcTmp);
					break;

				case CaptureTheFlag:
					m_xfChars.textOut(	pxdisplayDest,iPosX+74,iPosY+34,
										"f1  gamemode  [            ]");
					m_xfChars.textOut(	pxdisplayDest,iPosX+249,iPosY+34,
										"ctf");
					sprintf(	pcTmp,
								"    number of players  [ %1d ]",
								g_configData.m_iNrOfActivePlayers);
					m_xfChars.textOut(pxdisplayDest,iPosX+74,iPosY+74,pcTmp);
					break;

				case EatEmAll:
					m_xfChars.textOut(	pxdisplayDest,iPosX+74,iPosY+34,
										"f1  gamemode  [ eat'em all ]");
					sprintf(	pcTmp,
								"f2  number of players  [ %1d ]",
								g_configData.m_iNrOfActivePlayers);
					m_xfChars.textOut(pxdisplayDest,iPosX+74,iPosY+74,pcTmp);
					break;

				}


				m_xfChars.textOut(pxdisplayDest,iPosX+115,iPosY+114,"slow   speed   fast");
				m_xfChars.textOut(pxdisplayDest,iPosX+33,iPosY+134,
					"arrow left                 arrow right");

				m_xsSpeedBar.m_lX=iPosX+136;
				m_xsSpeedBar.m_lY=iPosY+132;
				m_xsSpeedBar.m_lFrame=0;
				m_xsSpeedBar.blit(pxdisplayDest);

				m_xsSpeedButton.m_lX=iPosX+144+((200-g_configData.m_iGameSpeed)/25)*16;
				m_xsSpeedButton.m_lY=iPosY+132;
				m_xsSpeedButton.m_lFrame=0;
				m_xsSpeedButton.blit(pxdisplayDest);


				m_xfChars.textOut(	pxdisplayDest,iPosX+92,iPosY+174,
									"f5  wrap around  [     ]");
				if (g_configData.m_bWrapAround)
					m_xfChars.textOut(pxdisplayDest,iPosX+267,iPosY+174,"on");
				else
					m_xfChars.textOut(pxdisplayDest,iPosX+263,iPosY+174,"off");


				m_xfChars.textOut(	pxdisplayDest,iPosX+51,iPosY+214,	
									"f6  wrap around color  [        ]");
				switch(g_configData.m_iWrapAroundTile)
				{
				case 64:
					m_xfChars.textOut(pxdisplayDest,iPosX+285,iPosY+214,"blue");
					break;
				case 65:
					m_xfChars.textOut(pxdisplayDest,iPosX+285,iPosY+214,"cyan");
					break;
				case 66:
					m_xfChars.textOut(pxdisplayDest,iPosX+280,iPosY+214,"green");
					break;
				case 67:
					m_xfChars.textOut(pxdisplayDest,iPosX+276,iPosY+214,"yellow");
					break;
				case 68:
					m_xfChars.textOut(pxdisplayDest,iPosX+289,iPosY+214,"red");
					break;
				}

				
				m_xfChars.textOut(	pxdisplayDest,iPosX+83,iPosY+248,
									"f9  show key configuration");
				m_xfChars.textOut(	pxdisplayDest,iPosX+92,iPosY+268,
									"f10  audio-video options");
				break;

			case 2:// Optionen Seite 3 (Audio/Video)
				m_xfChars.textOut(	pxdisplayDest,iPosX+87,iPosY+254,
									"f10  return to first page");
				m_xfChars.textOut(	pxdisplayDest,iPosX+155,iPosY+35,
									"sfx volume");
				m_xfChars.textOut(	pxdisplayDest,iPosX+96,iPosY+65,
									"f1 [               ] f2");
				iCountX=0;
				while(iCountX<g_configData.m_iSFXVolume)
				{
					m_xfChars.textOut(	pxdisplayDest,iPosX+129+iCountX*6,iPosY+65,"i");
					iCountX++;
				}

				m_xfChars.textOut(	pxdisplayDest,iPosX+146,iPosY+115,
									"music volume");
				m_xfChars.textOut(	pxdisplayDest,iPosX+96,iPosY+145,
									"f3 [               ] f4");
				iCountX=0;
				while(iCountX<g_configData.m_iMusicVolume)
				{
					m_xfChars.textOut(	pxdisplayDest,iPosX+129+iCountX*6,iPosY+145,"i");
					iCountX++;
				}

				m_xfChars.textOut(	pxdisplayDest,iPosX+92,iPosY+200,
									"f5  particle fx  [     ]");
				if (g_configData.m_bParticles)
					m_xfChars.textOut(pxdisplayDest,iPosX+267,iPosY+200,"on");
				else
					m_xfChars.textOut(pxdisplayDest,iPosX+263,iPosY+200,"off");


				break;

			default:// Optionen Seite 2 (Tastenkonfiguration)
				renderKeyConfig(pxdisplayDest,iPosX,iPosY+42);
				
				m_xfChars.textOut(	pxdisplayDest,iPosX+123,iPosY+15,
									"to redefine press");
				m_xfChars.textOut(	pxdisplayDest,iPosX+74,iPosY+35,
									"f1           f2           f3");

				m_xfChars.textOut(	pxdisplayDest,iPosX+92,iPosY+254,
									"f9  return to first page");

				break;
			}
			break;
		
		case 3://MenuPunkt Credits
			m_xfChars.textOut(pxdisplayDest,iPosX+133,iPosY+74,"code and gfx by");
			m_xfChars.textOut(pxdisplayDest,iPosX+129,iPosY+94,"christian roesch");
			m_xfChars.textOut(pxdisplayDest,iPosX+129,iPosY+134,"music and sfx by");
			m_xfChars.textOut(pxdisplayDest,iPosX+142,iPosY+154,"j. warthemann");
			m_xfChars.textOut(pxdisplayDest,iPosX+133,iPosY+194,"directx-wrapper");
			m_xfChars.textOut(pxdisplayDest,iPosX+173,iPosY+214,"xtreme");
			break;

		case 4://MenuPunkt Ende
			m_xfChars.textOut(pxdisplayDest,iPosX+114,iPosY+130,"press enter to quit");
			m_xfChars.textOut(pxdisplayDest,iPosX+164,iPosY+200,"(c) 2001");
			m_xfChars.textOut(pxdisplayDest,iPosX+164,iPosY+220,"tng-soft");
			break;
		}
	}


}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::processSubMenu
/////////////////////////////////////////////////////////////////////////////////////
// Info:	wertet Tastendruck fr das Untermenu iNr aus, unbearbeitet Tastendrcke
//			werden zurckgeliefert (sonst -1)
/////////////////////////////////////////////////////////////////////////////////////
int PPMenu::processSubMenu(int iNr)
{
	int iKey=analyzeKeys();
	
	switch	(iNr)
	{
	case 0: //Start Game
		switch (iKey)
		{
		case DIK_RETURN:
			m_stateNext=Game;
			break;

		case DIK_RIGHT:
			if (g_configData.m_gmActual!=SinglePlay ||
				g_configData.getLevelNr()<g_configData.m_iMapsFree )
				g_configData.getLevelNr(1);
			g_configData.loadLevel(&m_fieldPreview);
			m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
			g_xfmodHelper.playSample(m_pfsInc);
			break;

		case DIK_LEFT:
			g_configData.getLevelNr(-1);
			g_configData.loadLevel(&m_fieldPreview);
			m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
			g_xfmodHelper.playSample(m_pfsDec);
			break;
		}
		break;

	case 1: //HighScores
		switch (iKey)
		{
		case DIK_F1:
			g_xfmodHelper.playSample(m_pfsMisc);
			m_iChoosenState^=1;
			break;

		case DIK_RIGHT:
			g_xfmodHelper.playSample(m_pfsInc);
			if (m_iChoosenState < iNumberOfChallengeLevel*2)
				m_iChoosenState+=2;
			break;

		case DIK_LEFT:
			g_xfmodHelper.playSample(m_pfsDec);
			if (m_iChoosenState >= 2)
				m_iChoosenState-=2;
			break;
		}
		break;

	case 2: //Options
		switch(m_iChoosenState)
		{
		case 0:
/////////////////////////////////////////////////////////////////////////////////////
		// Optionen Seite 1
/////////////////////////////////////////////////////////////////////////////////////
			switch(iKey)
			{
				case DIK_RIGHT:
					g_configData.m_iGameSpeed-=25;
					g_xfmodHelper.playSample(m_pfsInc);
					if (g_configData.m_iGameSpeed<50)
						g_configData.m_iGameSpeed=50;
					break;
				case DIK_LEFT:
					g_configData.m_iGameSpeed+=25;
					g_xfmodHelper.playSample(m_pfsDec);
					if (g_configData.m_iGameSpeed>200)
						g_configData.m_iGameSpeed=200;
					break;

				case DIK_F5:
					g_configData.m_bWrapAround=!g_configData.m_bWrapAround;
					g_xfmodHelper.playSample(m_pfsMisc);
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
					break;
				case DIK_F6:
					g_configData.m_iWrapAroundTile++;
					g_xfmodHelper.playSample(m_pfsMisc);
					if(g_configData.m_iWrapAroundTile>68)
						g_configData.m_iWrapAroundTile=64;
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
					break;
				case DIK_F9:
					m_iChoosenState=1;
					g_xfmodHelper.playSample(m_pfsMisc);
					break;
				case DIK_F10:
					m_iChoosenState=2;
					g_xfmodHelper.playSample(m_pfsMisc);
					break;
			}

			switch (g_configData.m_gmActual)
			{
			case SinglePlay:
				g_configData.m_iNrOfActivePlayers=1;
				if (iKey==DIK_F1)
				{
					g_configData.m_gmActual=DeathMatch;
					g_xfmodHelper.playSample(m_pfsMisc);
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
				}
				break;

			case DeathMatch:
				if (iKey==DIK_F1)
				{
					g_configData.m_gmActual=Challenge;
					g_xfmodHelper.playSample(m_pfsMisc);
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
				}

				if (iKey==DIK_F2)
				{
					g_configData.m_iNrOfActivePlayers++;
					g_xfmodHelper.playSample(m_pfsMisc);
				}
				if (g_configData.m_iNrOfActivePlayers!=3 &&
					g_configData.m_iNrOfActivePlayers!=2)
					g_configData.m_iNrOfActivePlayers=2;
				break;
				
			case Challenge:
				g_configData.m_iNrOfActivePlayers=1;
				if (iKey==DIK_F1)
				{
					g_configData.m_gmActual=CaptureTheFlag;
					g_xfmodHelper.playSample(m_pfsMisc);
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
				}
				break;
				
			case CaptureTheFlag:
				g_configData.m_iNrOfActivePlayers=2;
				if (iKey==DIK_F1)
				{
					g_configData.m_gmActual=EatEmAll;
					g_xfmodHelper.playSample(m_pfsMisc);
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
				}
				break;

			case EatEmAll:
				if (iKey==DIK_F1)
				{
					g_configData.m_gmActual=SinglePlay;
					g_xfmodHelper.playSample(m_pfsMisc);
					g_configData.loadLevel(&m_fieldPreview);
					m_fieldPreview.render(&m_xsurfMapPreview,&m_xsSmallTileSet,0,0,3);
				}

				if (iKey==DIK_F2)
				{
					g_configData.m_iNrOfActivePlayers++;
					g_xfmodHelper.playSample(m_pfsMisc);
				}
				if (g_configData.m_iNrOfActivePlayers!=3 &&
					g_configData.m_iNrOfActivePlayers!=2)
					g_configData.m_iNrOfActivePlayers=2;
				break;
			}
			break;

		case 1:
/////////////////////////////////////////////////////////////////////////////////////
		// Optionen Seite 2 Keys
/////////////////////////////////////////////////////////////////////////////////////
			switch(iKey)
			{
			case DIK_F9:
				m_iChoosenState=0;
				g_xfmodHelper.playSample(m_pfsMisc);
				break;
			case DIK_F1:
				m_iChoosenState=1*16;
				g_xfmodHelper.playSample(m_pfsMisc);
				break;
			case DIK_F2:
				m_iChoosenState=2*16;
				g_xfmodHelper.playSample(m_pfsMisc);
				break;
			case DIK_F3:
				m_iChoosenState=3*16;
				g_xfmodHelper.playSample(m_pfsMisc);
				break;
			}
			break;

		case 2:
/////////////////////////////////////////////////////////////////////////////////////
		// Optionen Seite 3 (Audio)
/////////////////////////////////////////////////////////////////////////////////////
			switch(iKey)
			{
			case DIK_F10:
				m_iChoosenState=0;
				g_xfmodHelper.playSample(m_pfsMisc);
				break;
			case DIK_F1:
				g_configData.m_iSFXVolume--;
				if (g_configData.m_iSFXVolume<0) g_configData.m_iSFXVolume=0;
				FSOUND_SetSFXMasterVolume(11*g_configData.m_iSFXVolume);
				g_xfmodHelper.playSample(m_pfsDec);
				break;
			case DIK_F2:
				g_configData.m_iSFXVolume++;
				if (g_configData.m_iSFXVolume>23) g_configData.m_iSFXVolume=23;
				FSOUND_SetSFXMasterVolume(11*g_configData.m_iSFXVolume);
				g_xfmodHelper.playSample(m_pfsInc);
				break;
			case DIK_F3:
				g_configData.m_iMusicVolume--;
				if (g_configData.m_iMusicVolume<0) g_configData.m_iMusicVolume=0;
				g_xfmodHelper.m_iMusicVolume=11*g_configData.m_iMusicVolume;
				FMUSIC_SetMasterVolume(m_pfmMenuMuzak,11*g_configData.m_iMusicVolume);
				g_xfmodHelper.playSample(m_pfsDec);
				break;
			case DIK_F4:
				g_configData.m_iMusicVolume++;
				if (g_configData.m_iMusicVolume>23) g_configData.m_iMusicVolume=23;
				g_xfmodHelper.m_iMusicVolume=11*g_configData.m_iMusicVolume;
				FMUSIC_SetMasterVolume(m_pfmMenuMuzak,11*g_configData.m_iMusicVolume);
				if (g_configData.m_iMusicVolume==1)
					g_xfmodHelper.playModule(m_pfmMenuMuzak);
				g_xfmodHelper.playSample(m_pfsInc);
				break;
			case DIK_F5:
					g_configData.m_bParticles=!g_configData.m_bParticles;
					g_xfmodHelper.playSample(m_pfsMisc);
				break;
	
			}
			break;

			default:
/////////////////////////////////////////////////////////////////////////////////////
		// KeyConfig!
/////////////////////////////////////////////////////////////////////////////////////
			doKeyConfig(iKey);
			iKey=-1;
			break;

		}
		break;

	case 4: //Quit
		if (iKey==DIK_RETURN) m_stateNext=Outro;
		break;
	}

	return	iKey;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::renderKeyConfig
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt die aktuelle Tastenbelegung auf er Surface aus
/////////////////////////////////////////////////////////////////////////////////////
void PPMenu::renderKeyConfig(XDisplay* pxdisplayDest, int iPosX, int iPosY)
{
	const int	iHeadX=55;
	const int	iHeadY=15;
	const int	iDescX=10;
	const int	iDescY=50;
	const int	iDeltaX=114;
	const int	iDeltaY=25;

	char		pcEmpty[]="";
	char		pcTmp[12];
	int			iKey;

	int			iCountX;
	int			iCountY;
	int			iTmp;

	for (iCountX=0;iCountX<iNumberOfPlayers;iCountX++)
	{
		sprintf(pcTmp,"player %1d",iCountX+1);
		m_xfChars.textOut(pxdisplayDest,iPosX+iHeadX+iDeltaX*iCountX,iPosY+iHeadY,pcTmp);
	}

	for (iCountY=0;iCountY<6;iCountY++)
	{
		m_xsMoveIcons.m_lFrame=iCountY;
		m_xsMoveIcons.m_lX=iPosX+iDescX;
		m_xsMoveIcons.m_lY=iPosY+iDescY+iDeltaY*iCountY-4;
		m_xsMoveIcons.blit(pxdisplayDest);
	}

	for (iCountX=0;iCountX<iNumberOfPlayers;iCountX++)
		for (iCountY=0;iCountY<6;iCountY++)
		{
			//get key from Config class
			iKey=g_configData.getKey(iCountX,iCountY);

			//get Key-Name via DI
			if (iKey>=0 && iKey<256)
			{
				if(g_xiInput.getKeyboardText(iKey,pcTmp,12)!=DI_OK)
				{
					strcpy(pcTmp,"press a key"); //WTF has happenend? (inside keyconfig?)
				}

				//converting to small letters coz the font doesnt include capitals
				for (iTmp=0;pcTmp[iTmp]!=0;iTmp++)
				{
					if (pcTmp[iTmp]>='A' && pcTmp[iTmp]<='Z')
					{
						pcTmp[iTmp]+='a'-'A';
					}
				}
			}
			else
			{
				strcpy(pcTmp,pcEmpty);
			}

			//put Text on Screen
			m_xfChars.textOut(	pxdisplayDest,
								iPosX+iHeadX+iDeltaX*iCountX,
								iPosY+iDescY+iDeltaY*iCountY,
								pcTmp);
		}

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			PPMenu::doKeyConfig
/////////////////////////////////////////////////////////////////////////////////////
// Info:	sorgt fr die Neudefinition der Tasten, oder den Abbruch der 
//			Konfiguration bei ESC
/////////////////////////////////////////////////////////////////////////////////////
void PPMenu::doKeyConfig(int iKey)
{
	int iPlayer;
	int iKeyNr;
	int iOldKey;

	// Variabeln aus dem State extrahieren
	iPlayer=(m_iChoosenState/16)%16-1;
	iKeyNr=(m_iChoosenState/256)%16;
	iOldKey=(m_iChoosenState/4096)%256;

	//wenn noch nicht geschehen, alte Taste holen und in der KeyConfig lschen
	if (g_configData.getKey(iPlayer,iKeyNr)!=0)
	{
		iOldKey=g_configData.getKey(iPlayer,iKeyNr);
		g_configData.setKey(iPlayer,iKeyNr,0);
	}

	//neue Taste gedrckt? (ausser ESC und Pause)
	if (iKey!=DIK_ESCAPE && iKey!=DIK_PAUSE && iKey!=-1)
	{
		
		g_xfmodHelper.playSample(m_pfsMisc);
		g_configData.setKey(iPlayer,iKeyNr,iKey);
		iKeyNr++;
	}

	// Variabeln in den State kopieren
	m_iChoosenState=	16*(iPlayer+1)+
						256*iKeyNr+
						4096*iOldKey;

	//letzte Taste erreicht? => Abbruch
	if (iKeyNr==iNrOfKeys)
	{
		m_iChoosenState=1;
	}

	//Bei ESC alles abbrechen
	if (iKey==DIK_ESCAPE)
	{
		g_configData.setKey(iPlayer,iKeyNr,iOldKey);
		g_xfmodHelper.playSample(m_pfsMisc);
		m_iChoosenState=1;
	}
		
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	PPMenu.cpp
/////////////////////////////////////////////////////////////////////////////////////
