/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		Particle.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	26.09.2001
// letzte nderung:	26.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Diese Klasse modelliert simple Partikel			
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "Particle.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Particle::Particle
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
/////////////////////////////////////////////////////////////////////////////////////
Particle::Particle(void)
{
	m_fPosX=0;
	m_fPosY=0;
	m_fPosZ=0;
	m_fSpeedX=0;
	m_fSpeedY=0;
	m_fSpeedZ=0;

	m_iAge=iParticleMaxAge;

	m_iType=0; // 0= Partikel inaktiv
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Particle::create
/////////////////////////////////////////////////////////////////////////////////////
// Info:	erzeugt (oder verndert) den Partikel
//			wird als iType=0 bergeben wird der Partikel "ausgeschaltet"
/////////////////////////////////////////////////////////////////////////////////////
void Particle::create(	float fPosX,
						float fPosY,
						float fPosZ,
						float fSpeedX,
						float fSpeedY,
						float fSpeedZ,
						int iType)
{
	m_fPosX=fPosX;
	m_fPosY=fPosY;
	m_fPosZ=fPosZ;
	m_fSpeedX=fSpeedX;
	m_fSpeedY=fSpeedY;
	m_fSpeedZ=fSpeedZ;

	m_iAge=0;

	m_iType=iType; // 0= Partikel inaktiv

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Particle::update
/////////////////////////////////////////////////////////////////////////////////////
// Info:	bewegt den Partikel
/////////////////////////////////////////////////////////////////////////////////////
void Particle::update(int iTime)
{
	// is der berhaupt lebendig
	//if (m_iType<=0)
		//return;

	//Hhe aktualisieren
	m_fPosZ+=m_fSpeedZ*(float)iTime+fParticleGravity*iTime*iTime*0.5f;

	if (m_fPosZ<0)
	{ //liegt auf dem Boden, nicht weiterbewegen
		m_fPosZ=0;
		// doppelt so schnell altern
		m_iAge+=2*iTime;
		if (m_iAge>=iParticleMaxAge)
		{	//gestorben...
			m_iType=0;
		}
		return;
	}

	// altern
	m_iAge+=iTime;
	if (m_iAge>=iParticleMaxAge)
	{	//gestorben...
		m_iType=0;
		return;
	}


	//es geht abwrts *g*
	m_fSpeedZ+=fParticleGravity*iTime;

	// und die normale x/y-Bewegung
	m_fPosX+=m_fSpeedX*iTime;
	m_fPosY+=m_fSpeedY*iTime;


	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Particle::renderShadow
/////////////////////////////////////////////////////////////////////////////////////
// Info:	zeichnet den Schatten der Partikel (vor render aufrufen!)
/////////////////////////////////////////////////////////////////////////////////////
void Particle::renderShadow(XSurface* pxdDest, 
							XSprite* pxsParticle, 
							int iX, 
							int iY, 
							int iSize)
{
	int iScreenPosX;
	int iScreenPosY;

	// is der berhaupt lebendig
	if (m_iType<=0)
		return;

	iScreenPosX=iParticleOffset+iSize/2+iX;
	iScreenPosY=iParticleOffset+iSize/2+iY;

	iScreenPosX+=(int)(iSize*m_fPosX);
	iScreenPosY+=(int)(iSize*m_fPosY);

	pxsParticle->m_lFrame=(iParticleFrames*m_iAge)/iParticleMaxAge;
	pxsParticle->m_lX=iScreenPosX;
	pxsParticle->m_lY=iScreenPosY;
	pxsParticle->blit(pxdDest);
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Particle::render
/////////////////////////////////////////////////////////////////////////////////////
// Info:	zeichnet die Partikel 
/////////////////////////////////////////////////////////////////////////////////////
void Particle::render      (XSurface* pxdDest, 
							XSprite* pxsParticle, 
							int iX, 
							int iY, 
							int iSize)
{
	int iScreenPosX;
	int iScreenPosY;

	// is der berhaupt lebendig
	if (m_iType<=0)
		return;

	iScreenPosX=iParticleOffset+iSize/2+iX;
	iScreenPosY=iParticleOffset+iSize/2+iY;

	iScreenPosX+=(int)(iSize*m_fPosX);
	iScreenPosY+=(int)(iSize*m_fPosY);

	//Verschiebung durch Hhe
	iScreenPosY-=(int)(iSize*m_fPosZ);

	pxsParticle->m_lFrame=(iParticleFrames*m_iAge)/iParticleMaxAge
						+iParticleFrames*m_iType;

	pxsParticle->m_lX=iScreenPosX;
	pxsParticle->m_lY=iScreenPosY;
	pxsParticle->blit(pxdDest);
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Particle::active
/////////////////////////////////////////////////////////////////////////////////////
// Info:	gibt true zurck, wenn der Partikel aktiv ist
/////////////////////////////////////////////////////////////////////////////////////
bool Particle::active()
{
	if (m_iType>0)
		return true;

	return false;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	Particle.cpp
/////////////////////////////////////////////////////////////////////////////////////
