/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		Player.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	06.08.2001
// letzte nderung:	03.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Die Klasse kapselt die Daten eines Spielers und verwaltet seine Eingaben
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "Player.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Player::Player
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
//			der Spieler bekommt eine Nummer
/////////////////////////////////////////////////////////////////////////////////////
Player::Player(int iNr, int iStartX, int iStartY, int iDirection)
{
	m_iNr			=iNr;
	//ist die Nummer sinnvoll?
	if (m_iNr<0 || m_iNr>=iNumberOfPlayers) m_iNr=1;

	//Schlange fr den Spieler erzeugen
	m_pSnake=new Snake(	iStartX,
						iStartY,
						iDirection,
						g_TileSetData.getPlayerTileOffset(m_iNr));

	m_iPoints		=0;
	m_bPressedUp	=false;
	m_bPressedDown	=false;
	m_bPressedLeft	=false;
	m_bPressedRight	=false;
	m_bPressedCW	=false;
	m_bPressedCCW	=false;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Player::~Player
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Destructor
//
/////////////////////////////////////////////////////////////////////////////////////
Player::~Player()
{
	delete m_pSnake;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Player::checkInput
/////////////////////////////////////////////////////////////////////////////////////
// Info:	schaut welche Tasten gedrckt worden und welche losgelassen worden
//			(ich mach das mal ohne Schleife, wegen den Ausnahmen)
/////////////////////////////////////////////////////////////////////////////////////
void Player::checkInput()
{
	/////////////////////////////////////////////////////////////////////////////////
	//nach oben
	if (g_xiInput.m_keyboardData[g_configData.getKey(m_iNr,0)]!=0)
	{	//nach oben ist runtergedrckt
		if (!m_bPressedUp) //gerade gedrckt?
			m_pSnake->changeDirection(0);

		m_bPressedUp=true;
	}
	else
	{	//nach oben ist losgelassen
		m_bPressedUp=false;
	}

	/////////////////////////////////////////////////////////////////////////////////
	//nach rechts
	if (g_xiInput.m_keyboardData[g_configData.getKey(m_iNr,3)]!=0)
	{	//nach rechts ist runtergedrckt
		if (!m_bPressedRight) //gerade gedrckt?
			m_pSnake->changeDirection(1);

		m_bPressedRight=true;
	}
	else
	{	//nach rechts ist losgelassen
		m_bPressedRight=false;
	}

	/////////////////////////////////////////////////////////////////////////////////
	//nach unten
	if (g_xiInput.m_keyboardData[g_configData.getKey(m_iNr,1)]!=0)
	{	//nach unten ist runtergedrckt
		if (!m_bPressedDown) //gerade gedrckt?
			m_pSnake->changeDirection(2);

		m_bPressedDown=true;
	}
	else
	{	//nach unten ist losgelassen
		m_bPressedDown=false;
	}

	/////////////////////////////////////////////////////////////////////////////////
	//nach links
	if (g_xiInput.m_keyboardData[g_configData.getKey(m_iNr,2)]!=0)
	{	//nach links ist runtergedrckt
		if (!m_bPressedLeft) //gerade gedrckt?
			m_pSnake->changeDirection(3);

		m_bPressedLeft=true;
	}
	else
	{	//nach links ist losgelassen
		m_bPressedLeft=false;
	}

	/////////////////////////////////////////////////////////////////////////////////
	//drehen im Uhrzeigersinn
	if (g_xiInput.m_keyboardData[g_configData.getKey(m_iNr,5)]!=0)
	{	//turn cw ist runtergedrckt
		if (!m_bPressedCW) //gerade gedrckt?
			m_pSnake->changeDirection(-1);

		m_bPressedCW=true;
	}
	else
	{	//turn cw ist losgelassen
		m_bPressedCW=false;
	}

	/////////////////////////////////////////////////////////////////////////////////
	//drehen gegen den Uhrzeigersinn
	if (g_xiInput.m_keyboardData[g_configData.getKey(m_iNr,4)]!=0)
	{	//turn ccw ist runtergedrckt
		if (!m_bPressedCCW) //gerade gedrckt?
			m_pSnake->changeDirection(-2);

		m_bPressedCCW=true;
	}
	else
	{	//turn ccw ist losgelassen
		m_bPressedCCW=false;
	}
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			Player::addPoints
/////////////////////////////////////////////////////////////////////////////////////
// Info:	fgt dem PunkteKonto iPoints hinzu und gibt die Gesamtpunktzahl zurck
/////////////////////////////////////////////////////////////////////////////////////
int Player::addPoints(int iPoints)
{
	m_iPoints+=iPoints;
	return m_iPoints;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	Player.cpp
/////////////////////////////////////////////////////////////////////////////////////
