/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XFModHelper.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	09.09.2001
// letzte nderung:	28.09.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	hilft bei der Benutzung von FMod, kapselt die kritischen FMod-Funktionen			
//			um die Arbeit zur Fehlerbehandlungen zu vereinfachen. Es werden ebenfalls
//			sinnvolle DefaultParameter eingesetzt.
//			(Diese Klasse ist nicht dazu gedacht, FMod vollstndig zu kapseln...)
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "XFModHelper.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::XFModHelper
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Konstruktor, initialisiert alle Member der Klasse
/////////////////////////////////////////////////////////////////////////////////////
XFModHelper::XFModHelper(void)
{
	m_bInitialized=false;

	m_iSFXVolume=255;
	m_iMusicVolume=255;

	m_iSWChannels=0;

	FSOUND_SetSFXMasterVolume(255);

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::~XFModHelper
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Destruktor, schliesst die FMod-Engine 
/////////////////////////////////////////////////////////////////////////////////////
XFModHelper::~XFModHelper(void)
{
	if (m_bInitialized)
		uninitialize();
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::uninitilialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Schliesst die FMod-Engine
/////////////////////////////////////////////////////////////////////////////////////
void XFModHelper::uninitialize()
{
	FSOUND_Close();
	m_bInitialized=false;

}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:	initiiert die FMOD-Engine
//			bei Erfolg wird true zurckgegeben
/////////////////////////////////////////////////////////////////////////////////////
bool XFModHelper::initialize(	int iOutputType/* =FSOUND_OUTPUT_DSOUND */,
								int iDriver/* =0 */,
								int iMixRate/* =44100 */,
								int iMaxSoftwareChannels/* =32 */,
								unsigned int uiFlags/* =0 */)
{
	m_iSWChannels=iMaxSoftwareChannels;

	if (m_bInitialized)	
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::initialize > (WARNUNG!!!) war bereits initialisiert !\n");
		#endif
		uninitialize();
	}

	if (FSOUND_GetVersion() < FMOD_VERSION)
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::initialize > (FEHLER!!!) Falsche FMOD Version !!!\n");
		#endif
		return false;
	}

	if (iOutputType!=FSOUND_OUTPUT_DSOUND)
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::initialize > versuche spezifizierten Output zu setzen...\n");
		#endif

		if (FSOUND_SetOutput(iOutputType))
		{
			if (!FSOUND_SetDriver(iDriver))
			{
				#if defined(XDEBUG) || defined(XFMODHELPER)
				XLogging::log("XFModHelper::initialize > (FEHLER!!!) Treiber ungltig !!!\n");
				XLogging::log("FMOD:");
				XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
				XLogging::log("\n");
				#endif
				return false;
			}

			if(FSOUND_Init(iMixRate,iMaxSoftwareChannels,uiFlags))
			{
				m_bInitialized=true;
					#if defined(XDEBUG) || defined(XFMODHELPER)
					XLogging::log("XFModHelper::initialize > ...erfolgreich\n");
					#endif
				return true;
			}
		}
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::initialize > ...failed\n");
		XLogging::log("FMOD:");
		XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
		XLogging::log("\n");
		#endif
	}


	#if defined(XDEBUG) || defined(XFMODHELPER)
	XLogging::log("XFModHelper::initialize > versuche DirectSound zu setzen...\n");
	#endif

	if (FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND))
	{
		if (!FSOUND_SetDriver(0))
		{
			#if defined(XDEBUG) || defined(XFMODHELPER)
			XLogging::log("XFModHelper::initialize > (FEHLER!!!) Treiber ungltig !!!\n");
			XLogging::log("FMOD:");
			XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
			XLogging::log("\n");
			#endif
			return false;
		}

		if(FSOUND_Init(iMixRate,iMaxSoftwareChannels,uiFlags))
		{
			m_bInitialized=true;
				#if defined(XDEBUG) || defined(XFMODHELPER)
				XLogging::log("XFModHelper::initialize > ...erfolgreich\n");
				#endif
			return true;
		}
	}

	#if defined(XDEBUG) || defined(XFMODHELPER)
	XLogging::log("XFModHelper::initialize > ...failed\n");
	XLogging::log("FMOD:");
	XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
	XLogging::log("\n");
	XLogging::log("XFModHelper::initialize > versuche WinMM zu setzen...\n");
	#endif

	if (FSOUND_SetOutput(FSOUND_OUTPUT_WINMM))
	{
		if (!FSOUND_SetDriver(0))
		{
			#if defined(XDEBUG) || defined(XFMODHELPER)
			XLogging::log("XFModHelper::initialize > (FEHLER!!!) Treiber ungltig !!!\n");
			XLogging::log("FMOD:");
			XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
			XLogging::log("\n");
			#endif
			return false;
		}

		if(FSOUND_Init(iMixRate,iMaxSoftwareChannels,uiFlags))
		{
			m_bInitialized=true;
				#if defined(XDEBUG) || defined(XFMODHELPER)
				XLogging::log("XFModHelper::initialize > ...erfolgreich\n");
				#endif
			return true;
		}
	}

	#if defined(XDEBUG) || defined(XFMODHELPER)
	XLogging::log("XFModHelper::initialize > ...failed\n");
	XLogging::log("FMOD:");
	XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
	XLogging::log("\n");
	XLogging::log("XFModHelper::initialize > versuche NoSound zu setzen...\n");
	#endif

	if (FSOUND_SetOutput(FSOUND_OUTPUT_NOSOUND))
	{
		if (!FSOUND_SetDriver(0))
		{
			#if defined(XDEBUG) || defined(XFMODHELPER)
			XLogging::log("XFModHelper::initialize > (FEHLER!!!) Treiber ungltig !!!\n");
			XLogging::log("FMOD:");
			XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
			XLogging::log("\n");
			#endif
			return false;
		}

		if(FSOUND_Init(iMixRate,iMaxSoftwareChannels,uiFlags))
		{
			m_bInitialized=true;
				#if defined(XDEBUG) || defined(XFMODHELPER)
				XLogging::log("XFModHelper::initialize > ...erfolgreich\n");
				#endif
			return true;
		}
	}

	#if defined(XDEBUG) || defined(XFMODHELPER)
	XLogging::log("XFModHelper::initialize > ...failed\n");
	XLogging::log("FMOD:");
	XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
	XLogging::log("\n");
	#endif

	uninitialize();
	return false;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::loadSample
/////////////////////////////////////////////////////////////////////////////////////
// Info:	ldt ein Sample (*.wav) und gibt Weiger auf ihn zurck
/////////////////////////////////////////////////////////////////////////////////////
FSOUND_SAMPLE* XFModHelper::loadSample(	const char *pcName, 
										int iIndex/* =FSOUND_FREE */, 
										unsigned int iMode/* =FSOUND_NORMAL */)
{
	FSOUND_SAMPLE* pfsampleTmp;
	
	pfsampleTmp=FSOUND_Sample_Load(	iIndex,
									pcName,
									iMode,
									0);

	if (pfsampleTmp==NULL)
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::loadSample > (FEHLER!!!) File '");
		XLogging::log(pcName);
		XLogging::log("' konnte nicht geladen werden.\n");
		XLogging::log("FMOD:");
		XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
		XLogging::log("\n");
		#endif
	}
	else
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::loadSample > File '");
		XLogging::log(pcName);
		XLogging::log("' geladen.\n");
		#endif
	}

	return pfsampleTmp;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::loadModule
/////////////////////////////////////////////////////////////////////////////////////
// Info:	ldt einen Song (*.mod,*.s3m,*.mid,...) und gibt einen Zeiger auf ihn 
//			zurck
/////////////////////////////////////////////////////////////////////////////////////
FMUSIC_MODULE* XFModHelper::loadModule(const char *pcName)
{
	FMUSIC_MODULE* pfmodTmp;
	
	pfmodTmp = FMUSIC_LoadSong(pcName);

	if (pfmodTmp==NULL)
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::loadModule > (FEHLER!!!) File '");
		XLogging::log(pcName);
		XLogging::log("' konnte nicht geladen werden.\n");
		XLogging::log("FMOD:");
		XLogging::log(FMOD_ErrorString(FSOUND_GetError()));
		XLogging::log("\n");
		#endif
	}
	else
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::loadModule > File '");
		XLogging::log(pcName);
		XLogging::log("' geladen.\n");
		#endif
	}

	return pfmodTmp;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::playSample
/////////////////////////////////////////////////////////////////////////////////////
// Info:	spielt das Sample auf das gezeigt wird, es kann eine Panning Position
//			oder ein konkreter Channel gewhlt werden, zurckgegeben wir der 
//			tatschlich benutze Channel
//											iPan=0 ganz links / iPan=255 ganz rechts
/////////////////////////////////////////////////////////////////////////////////////
int XFModHelper::playSample(	FSOUND_SAMPLE* psampleDest, 
								int iPan/* =128 */, 
								int iChannel/* =FSOUND_FREE */)
{
	int iTmp;

		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::playSample > Playing\n");
		#endif

	if (m_iSFXVolume==0)
		return -1;

	iTmp=FSOUND_PlaySound( iChannel, psampleDest);
	
	if (iTmp==-1)
	{
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::playSample > No Channel Left: ");
		XLogging::log(FSOUND_GetChannelsPlaying());
		XLogging::log(" of ");
		XLogging::log(FSOUND_GetMaxChannels());
		XLogging::log(" Channels playing!\n");
		#endif
		return iTmp;
	}
	
	FSOUND_SetVolume(iTmp,m_iSFXVolume);

	FSOUND_SetPan(iTmp,iPan);
		#if defined(XDEBUG) || defined(XFMODHELPER)
		XLogging::log("XFModHelper::playSample > ");
		XLogging::log(FSOUND_GetChannelsPlaying());
		XLogging::log(" of ");
		XLogging::log(FSOUND_GetMaxChannels());
		XLogging::log(" Channels playing!\n");
		#endif
	return iTmp;
}

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFModHelper::playModule
/////////////////////////////////////////////////////////////////////////////////////
// Info:	spielt die Musik ab, dabei werden iChannelsNotToUse nicht benutzt,
//			wenn die Lautstrke im Modul < als iMinVolume ist, d.h. diese Anzahl
//			steht fr SFX zur Verfgung. Die Lautstrke in mods ist stets 0 - 64.
/////////////////////////////////////////////////////////////////////////////////////
int XFModHelper::playModule(FMUSIC_MODULE* pmusicDest,
							int iChannelsNotToUse/* =8 */,
							int iMinVolume/* =32 */)
{
	int iTmp;

	if (m_iMusicVolume==0)
		return -1;

	FMUSIC_SetMasterVolume(pmusicDest,m_iMusicVolume);
	FMUSIC_OptimizeChannels(pmusicDest,
							m_iSWChannels-iChannelsNotToUse,
							iMinVolume);
	
	iTmp=FMUSIC_PlaySong(pmusicDest);

	return iTmp;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XFModHelper.cpp
/////////////////////////////////////////////////////////////////////////////////////
