/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XFont.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak
//
// Erstelldatum:	30.08.2000
// letzte nderung:	15.03.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Die Klasse XFont verwaltet einen bitmapbasierten Zeichensatz.
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

#define DIRECTDRAW_VERSION 0x0700 // Dann klappts auch mit dem "DirectX>7-SDK"-Nachbarn

// XGame-includes
#include "XFont.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::XFont
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor von XFont, initialisiert alle Datenmember der
//					Klasse.
/////////////////////////////////////////////////////////////////////////////////////

XFont::XFont(void)
{
	resetData();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::resetData
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt alle Datenmember der Klasse zurueck
/////////////////////////////////////////////////////////////////////////////////////

void XFont::resetData(void)
{
	m_lHeight					= 0;
	m_lNrOfChars				= 0;
	m_lpstrFilename				= NULL;
	m_lpstrSubstitutionString	= NULL;
	m_plKerning					= NULL;
	m_plWidth					= NULL;
	m_bIsInitialized			= false;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::~XFont
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor von XFont, ruft XFont::uninitialize auf
/////////////////////////////////////////////////////////////////////////////////////

XFont::~XFont(void)
{
	// Die Pruefung, ob das Objekt initialisiert ist, findet schon hier statt damit
	// keine WARNUNG ausgegeben wird, wenn ein Objekt erzeugt, aber nie verwendet
	// wird. Siehe hierzu auch XSprite::uninitialize.

	if (m_bIsInitialized==true)
	{
		uninitialize();
	}
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			erzeugt einen neuen Font gemaess den Parametern der Datei
//					lpstrFontfile
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFont::initialize(XDisplay*	pXDisplay,
						  LPSTR		lpstrFontfile)
{
	HRESULT		hDDResult;
	const int	STRLEN = 1024;
	char		cReturned[STRLEN];
	HFILE		hFile;
	OFSTRUCT	ReOpenBuff;
	DDCOLORKEY	DDColorKey;
	char		cSeparators[] = ", ";
	char*		pcToken;
	long		lCount;
	WORD		wRGB[3]			= {0,0,0};

	#if defined(XDEBUG) || defined(XFONTDEBUG)
	if (m_bIsInitialized==true)
	{
		XLogging::log("XFont::initialize > (FEHLER!!!) Font bereits initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if ((!pXDisplay) || (!pXDisplay->m_lpDirectDraw7))
	{
		XLogging::log("XFont::initialize > (FEHLER!!!) DirectDraw7-Objekt existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif

	// Pruefen, ob Datei vorhanden
	hFile = OpenFile(lpstrFontfile,&ReOpenBuff,OF_EXIST);
	
	if (hFile==HFILE_ERROR)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Fontdatei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" nicht gefunden\n");
		#endif

		return DDERR_NOTFOUND;
	}
	
	// ColorKey holen
	GetPrivateProfileString("XFont","ColorKey","",cReturned,STRLEN,lpstrFontfile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Schlssel ColorKey in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		return DDERR_NOTFOUND;
	}
	lCount	= 0;
	pcToken = strtok(cReturned,cSeparators);
	
	while ((pcToken!=NULL) && (lCount<3))
	{
		wRGB[lCount]	= (WORD) atol(pcToken);	
		lCount++;
		pcToken	= strtok(NULL,cSeparators);
	}

	if (lCount!=3)
	{
		#if defined(XDEBUG) || defined(XSPRITEDEBUG)
		XLogging::log("XFont::initialize > (WARNUNG!!!) ColorKey-Daten ungltig \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\"\n");
		#endif
	}

	DDColorKey.dwColorSpaceHighValue	=
	DDColorKey.dwColorSpaceLowValue		= XDisplay::convertRGBToColorModel(wRGB[0],wRGB[1],wRGB[2]);

	// FontBitmap holen
	GetPrivateProfileString("XFont","FontBitmap","",cReturned,STRLEN,lpstrFontfile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Schlssel FontBitmap in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		return DDERR_NOTFOUND;
	}
	
	// Surface mit Fontdaten fuellen
	hDDResult = m_XSurfaceCharset.initialize(pXDisplay,cReturned,&DDColorKey);

	// Initialisierung erfolgt
	m_bIsInitialized	= true;

	// Dateinamen kopieren
	m_lpstrFilename = new char[strlen(lpstrFontfile)+1];
	if (!m_lpstrFilename)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Zuwenig Speicher fr Filename\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_OUTOFMEMORY;
	}
	strcpy(m_lpstrFilename,lpstrFontfile);

	// Hoehe holen
	GetPrivateProfileString("XFont","Height","",cReturned,STRLEN,lpstrFontfile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Schlssel Height in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}

	m_lHeight = atol(cReturned);
	
	// SubstitutionString holen
	GetPrivateProfileString("XFont","SubstitutionString","",cReturned,STRLEN,lpstrFontfile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Schlssel SubstitutionString in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}
	
	m_lpstrSubstitutionString = new char[strlen(cReturned)+1];

	if (!m_lpstrSubstitutionString)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Zuwenig Speicher fr SubstitutionString\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_OUTOFMEMORY;
	}

	strcpy(m_lpstrSubstitutionString,cReturned);
	
	m_lNrOfChars = strlen(m_lpstrSubstitutionString);

	// Speicher fuer plWidth allokieren
	m_plWidth = new long[m_lNrOfChars];

	if (!m_plWidth)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Zuwenig Speicher fr Widths-Array\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_OUTOFMEMORY;
	}
	for (lCount=0;lCount<m_lNrOfChars;lCount++)
		m_plWidth[lCount]=0;

	// Speicher fuer plKerning allokieren
	m_plKerning = new long[m_lNrOfChars];

	if (!m_plKerning)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Zuwenig Speicher fr Kerning-Array\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_OUTOFMEMORY;
	}
	for (lCount=0;lCount<m_lNrOfChars;lCount++)
		m_plKerning[lCount]=0;

	// Zeichenbreiten holen
	GetPrivateProfileString("XFont","Widths","",cReturned,STRLEN,lpstrFontfile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Schlssel Widths in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}
	
	lCount	= 0;
	pcToken = strtok(cReturned,cSeparators);
	
	while ((pcToken!=NULL) && (lCount<m_lNrOfChars))
	{
		m_plWidth[lCount]=atol(pcToken);	
		lCount++;
		pcToken = strtok(NULL,cSeparators);
	}
	
	if (lCount!=m_lNrOfChars)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (WARNUNG!!!) Anzahl Breiten in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" zu klein\n");
		#endif
	}

	// Zeichenkernings holen
	GetPrivateProfileString("XFont","Kernings","",cReturned,STRLEN,lpstrFontfile);

	if (!strlen(cReturned))
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (FEHLER!!!) Schlssel Kernings in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" existiert nicht\n");
		#endif

		_lclose(hFile);

		uninitialize();

		return DDERR_NOTFOUND;
	}
	
	lCount	= 0;
	pcToken = strtok(cReturned,cSeparators);
	
	while ((pcToken!=NULL) && (lCount<m_lNrOfChars))
	{
		m_plKerning[lCount]=atol(pcToken);	
		lCount++;
		pcToken = strtok(NULL,cSeparators);
	}
	
	if (lCount!=m_lNrOfChars)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::initialize > (WARNUNG!!!) Anzahl Kerningangaben in Datei \"");
		XLogging::log(lpstrFontfile);
		XLogging::log("\" zu klein\n");
		#endif
	}

	#if defined(XDEBUG) || defined(XFONTDEBUG)
	XLogging::log("XFont::initialize > Font erzeugt \"");
	XLogging::log(lpstrFontfile);
	XLogging::log("\"\n");
	#endif

	_lclose(hFile);

	return hDDResult;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt den Font wieder frei
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFont::uninitialize(void)
{
	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::uninitialize > (WARNUNG!!!) Font nicht initialisiert\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}
	
	m_XSurfaceCharset.uninitialize();

	#if defined(XDEBUG) || defined(XFONTDEBUG)
	XLogging::log("XFont::uninitialize > Font freigegeben \"");
	XLogging::log(m_lpstrFilename);
	XLogging::log("\"\n");
	#endif

	if (m_lpstrFilename)
	{
		delete [] m_lpstrFilename;
		m_lpstrFilename	= NULL;
	}

	if (m_lpstrSubstitutionString)
	{
		delete [] m_lpstrSubstitutionString;
		m_lpstrSubstitutionString	= NULL;
	}

	if (m_plKerning)
	{
		delete [] m_plKerning;
		m_plKerning	= NULL;
	}

	if (m_plWidth)
	{
		delete [] m_plWidth;
		m_plWidth	= NULL;
	}
	
	// Member ruecksetzen
	resetData();

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::textOut
/////////////////////////////////////////////////////////////////////////////////////
// Info:			gibt einen String auf die Surface lpDestinationXSurface aus
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFont::textOut(XSurface*	lpDestinationXSurface,
					   long			lX,
					   long			lY,
					   LPSTR		lpstrText,
					   DWORD		dwFlags)
{
	HRESULT	res = DD_OK;
	long	lCursorPositionX,
			lI,
			lJ;
	RECT	SrcRect;
	
	lCursorPositionX = lX;

	#if defined(XDEBUG) || defined(XFONTDEBUG)
	if (m_bIsInitialized==false)
	{
		XLogging::log("XFont::textOut > (FEHLER!!!) Font nicht initialisiert\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if (!lpDestinationXSurface)
	{
		XLogging::log("XFont::textOut > (FEHLER!!!) Ziel-Surface existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif

	for (lI=0;lI<(long) strlen(lpstrText);lI++)
	{
		lJ = 0;

		SrcRect.top		=
		SrcRect.left	= 0;

		while ((lJ<m_lNrOfChars) && (m_lpstrSubstitutionString[lJ]!=lpstrText[lI]))
		{
			SrcRect.left	+= m_plWidth[lJ++];
		}
		
		if (lJ!=m_lNrOfChars)
		{
			SrcRect.top	   += int (SrcRect.left/m_XSurfaceCharset.getWidth())*m_lHeight;
			SrcRect.bottom	= SrcRect.top+m_lHeight;
			SrcRect.left	= SrcRect.left%m_XSurfaceCharset.getWidth();
			SrcRect.right	= SrcRect.left+m_plWidth[lJ];
			
			res = m_XSurfaceCharset.blit(lpDestinationXSurface,lCursorPositionX,lY,&SrcRect,dwFlags);
			
			lCursorPositionX += m_plKerning[lJ];
		}
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		else
		{
			char	c[2]	= "@";
			
			c[0]	= lpstrText[lI];
			XLogging::log("XFont::textOut > (WARNUNG!!!) Zeichen \"");
			XLogging::log(c);
			XLogging::log("\" in Font \"");
			XLogging::log(m_lpstrFilename);
			XLogging::log("\" existiert nicht\n");
		}
		#endif
	}	

	return res;		 
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFont::restore
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Restauriert einen verlorenen Font. Laedt auch die Bitmap-Daten
//					neu.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFont::restore(void)
{
	HRESULT				hDDResult;

	if (m_bIsInitialized==false)
	{
		#if defined(XDEBUG) || defined(XFONTDEBUG)
		XLogging::log("XFont::restore > (FEHLER!!!) Font nicht initialisiert\n");
		#endif
		
		return DDERR_INVALIDOBJECT;
	}

	// Surface restoren,...
	hDDResult	= m_XSurfaceCharset.restore();

	#if defined(XDEBUG) || defined(XFONTDEBUG)
	if (hDDResult!=DD_OK)
	{
		XLogging::log("XFont::restore > (FEHLER!!!) restore fehlgeschlagen\n");
	}
	else
	{
		XLogging::log("XFont::restore > Font restored \"");
		XLogging::log(m_lpstrFilename);
		XLogging::log("\"\n");
	}
	#endif

	return hDDResult;
}

/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XFont.cpp
/////////////////////////////////////////////////////////////////////////////////////
