/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XFrameCounter.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			pHreak (pHreak@x-treme-online.de)
//
// Erstelldatum:	16.03.2001
// letzte Aenderung:16.03.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Die Klasse XFrameCounter repraesentiert einen FPS-Zaehler. Die
//					blit-Methode muss pro Frame GENAU EINMAL aufgerufen werden.
//					Mit setText() kann ein Ausgabetext angegeben werden, in dem $f<n>
//					durch den FPS-Wert ersetzt wird. n bestimmt die Anzahl Ziffern,
//					die dabei Verwendung finden (2 <= n <= 9).
//					Bsp.: "fps $f3" fuehrt zu dieser Ausgabe: "fps 025"
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000/2001 by x-treme
/////////////////////////////////////////////////////////////////////////////////////

// includes
#include "XFrameCounter.h"

// Methoden-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::XFrameCounter
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Konstruktor
/////////////////////////////////////////////////////////////////////////////////////

XFrameCounter::XFrameCounter()
{
	resetData();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::resetData
/////////////////////////////////////////////////////////////////////////////////////
// Info:			setzt alle Datenmember der Klasse zurueck
/////////////////////////////////////////////////////////////////////////////////////

void XFrameCounter::resetData(void)
{
	strcpy(m_cStringOut,"");
	m_dwLastTicks		= 0;
	m_lX				= 0;
	m_lY				= 0;
	m_pcText			= NULL;
	m_pxfFont			= NULL;
	m_ulFramesPerSecond	= 0;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::~XFrameCounter
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Destruktor
/////////////////////////////////////////////////////////////////////////////////////

XFrameCounter::~XFrameCounter()
{
	uninitialize();
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			deinitialisiert die Klasse
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFrameCounter::uninitialize(void)
{
	if (m_pcText)
	{
		delete [] m_pcText;
		m_pcText	= NULL;
	}
	
	// Datenmember zuruecksetzen
	resetData();

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::setText
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Setzt den auszugebenen Text
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFrameCounter::setText(char	*pcText)
{
	if (m_pcText)
	{
		delete [] m_pcText;
		m_pcText	= NULL;
	}

	// Dateinamen kopieren
	m_pcText = new char[strlen(pcText)+1];
	if (!m_pcText)
	{
		#if defined(XDEBUG) || defined(XFRAMECOUNTERDEBUG)
		XLogging::log("XFrameCounter::setText > (FEHLER!!!) Zuwenig Speicher fr Text\n");
		#endif

		return DDERR_OUTOFMEMORY;
	}
	strcpy(m_pcText,pcText);

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::setFont
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Setzt den bei der Ausgabe zu verwendenden XFont
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFrameCounter::setFont(XFont	*pxfFont)
{
	if (!pxfFont)
	{
		#if defined(XDEBUG) || defined(XFRAMECOUNTERDEBUG)
		XLogging::log("XFrameCounter::setFont > (FEHLER!!!) Zeichensatz ungltig\n");
		#endif

		return DDERR_INVALIDOBJECT;
	}
	m_pxfFont	= pxfFont;

	return DD_OK;
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			XFrameCounter::blit
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Gibt den Text m_pcText auf der Surface pDestinationXSurface aus.
//					GENAU EINMAL pro Frame aufrufen !!!
/////////////////////////////////////////////////////////////////////////////////////

HRESULT XFrameCounter::blit(XSurface	*pDestinationXSurface,
							DWORD		dwFlags)
{
	DWORD	dwTicks;

	#if defined(XDEBUG) || defined(XFRAMECOUNTERDEBUG)
	if (!m_pcText)
	{
		XLogging::log("XFrameCounter::blit > (FEHLER!!!) Kein Text angegeben\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if (!m_pxfFont)
	{
		XLogging::log("XFrameCounter::blit > (FEHLER!!!) Kein Font angegeben\n");
		
		return DDERR_INVALIDOBJECT;
	}
	if (!pDestinationXSurface)
	{
		XLogging::log("XFrameCounter::blit > (FEHLER!!!) Ziel-Surface existiert nicht\n");
		
		return DDERR_INVALIDOBJECT;
	}
	#endif

	// einen Frame dazu
	m_ulFramesPerSecond++;

	if ((dwTicks=GetTickCount())-m_dwLastTicks>1000)
	{ // eine Sekunde ist vergangen
		m_dwLastTicks		= dwTicks;

		// Ausgabe-String zusammenbauen
		char	cFormatString[5]	= "%.#d";
		char	cFPSString[10];
		char	*pcPosOfFPS;
		char	*pcActualPos		= m_pcText;

		// String auf Laenge 0 setzen
		m_cStringOut[0]	= '\0';

		while ((pcPosOfFPS=strstr(pcActualPos,"$f"))!=NULL)
		{
			// String-Teil bis zu $f anhaengen
			strncat(m_cStringOut,pcActualPos,pcPosOfFPS-pcActualPos);

			// Format-String modifizieren
			cFormatString[2]	= pcPosOfFPS[2];

			sprintf(cFPSString,cFormatString,m_ulFramesPerSecond);
			strcat(m_cStringOut,cFPSString);

			// $f<n> ueberspringen (= 3 Zeichen)
			pcActualPos	= pcPosOfFPS+3;
		}
		strcat(m_cStringOut,pcActualPos);

		// von Neuem Frames zaehlen
		m_ulFramesPerSecond	= 0;
	}

	return m_pxfFont->textOut(pDestinationXSurface,m_lX,m_lY,m_cStringOut);
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XFrameCounter.cpp
/////////////////////////////////////////////////////////////////////////////////////