/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		XLogging.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Baal
//
// Erstelldatum:	12.09.2000
// letzte nderung:	17.08.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Klasse XLogging, um in den Debugger-Output und/oder in eine Datei
//					zu loggen
/////////////////////////////////////////////////////////////////////////////////////
// Copyright 2000 by x-treme
/////////////////////////////////////////////////////////////////////////////////////


// XGame-includes
#include "XLogging.h"

ofstream	XLogging::out;

char		XLogging::m_cFileName[_MAX_PATH]	= "XLogging.log";

bool		XLogging::m_bDebug					= true;
bool		XLogging::m_bFile					= false;
bool		XLogging::m_bAppend					= false;
bool		XLogging::m_bFileCreated			= false;


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::setContext
///////////////////////////////////////////////////////////////////////////////////////
// Info:			Bestimmt Output-Art
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::setContext(bool bDebug, 
							bool bFile, 
							bool bAppend, 
							char* lpFileName)
{
	m_bDebug		= bDebug;
	m_bFile			= bFile;
	m_bAppend		= bAppend;

	strncpy(m_cFileName, lpFileName, _MAX_PATH);
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::getContext
///////////////////////////////////////////////////////////////////////////////////////
// Info:			Besorgt Output-Art
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::getContext(bool &bDebug, bool &bFile, bool &bAppend)
{
	bDebug	= m_bDebug;
	bFile	= m_bFile;
	bAppend	= m_bAppend;
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::openFile
///////////////////////////////////////////////////////////////////////////////////////
// Info:			oeffnet (falls noetig) die Datei, in die geschrieben wird
///////////////////////////////////////////////////////////////////////////////////////

bool XLogging::openFile(void)
{
	out.open(m_cFileName, ios::out | ios::app);

	if (out.is_open() == 0)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::openFile > (ERROR!!!) could not open file\n");
		#endif

		return false;
	}
	
	return true;
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::closeFile
///////////////////////////////////////////////////////////////////////////////////////
// Info:			schliesst (falls noetig) die Datei, in die geschrieben wurde
///////////////////////////////////////////////////////////////////////////////////////

bool XLogging::closeFile(void)
{
	out.close();

	if (out.is_open() != 0)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::openFile > (ERROR!!!) could not close file\n");
		#endif

		return false;
	}

	return true;
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::clearFile
///////////////////////////////////////////////////////////////////////////////////////
// Info:			loescht den Inhalt der Datei
///////////////////////////////////////////////////////////////////////////////////////

bool XLogging::clearFile(void)
{
	out.open(m_cFileName, ios::trunc);

	if (out.is_open() == 0)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::clearFile > (ERROR!!!) could not clear file\n");
		#endif

		return false;
	}

	out.close();
	
	return true;
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::getTime
///////////////////////////////////////////////////////////////////////////////////////
// Info:			ermittelt die aktuelle Zeit
///////////////////////////////////////////////////////////////////////////////////////

bool XLogging::getTime(char* pTime, int nTimeLength)
{
	int count;
	char dateBuffer[20];
	char timeBuffer[20];
	char* string;
	SYSTEMTIME SystemTime;	

	// Datum und Zeit ermitteln
	GetLocalTime(&SystemTime);

	// Datum formatieren
	count = GetDateFormat(LOCALE_SYSTEM_DEFAULT, DATE_SHORTDATE, &SystemTime, NULL, dateBuffer, sizeof(dateBuffer));

	if(count == 0)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::getTime > (ERROR!!!) could not get date\n");
		#endif

		return false;
	}

	count = GetTimeFormat(LOCALE_SYSTEM_DEFAULT, TIME_NOSECONDS | TIME_NOTIMEMARKER, &SystemTime, NULL, timeBuffer, sizeof(timeBuffer));

	if(count == 0)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::getTime > (ERROR!!!) could not get time\n");
		#endif

		return false;
	}
	
	string = new char [strlen(dateBuffer) + strlen(" / ") + strlen(timeBuffer) + strlen(" : ") + 1];

	strcpy(string, dateBuffer);
	strcat(string, " / ");
	strcat(string, timeBuffer);
	strcat(string, " : ");

	strncpy(pTime, string, nTimeLength);
	pTime[nTimeLength - 1] = '\0';

	delete string;
	
	return true;
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::logTime
///////////////////////////////////////////////////////////////////////////////////////
// Info:			schreibt die ermittelte Zeit in den spezifizierten Context
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::logTime(void)
{
	char cTime[30];

	if (getTime(cTime, sizeof(cTime)) == true)
	{
		if (openFile() == true)
		{
			out << cTime;

			closeFile();
		}
	}
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::logInDebugger
///////////////////////////////////////////////////////////////////////////////////////
// Info:			loggt String in Debugger
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::logInDebugger(const char* pMessage)
{
	OutputDebugString(pMessage);
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::logInFile
///////////////////////////////////////////////////////////////////////////////////////
// Info:			loggt String in File
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::logInFile(const char* pMessage)
{
	if (openFile() == true)
	{
		out << pMessage;
		closeFile();
	}
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::log
///////////////////////////////////////////////////////////////////////////////////////
// Info:			schreibt in den spezifizierten Context
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::log(const char* pMessage, bool bTimeStamp)
{
	// hier findet das eigentliche logging statt

	// logging in Debugger UND File
	if ((m_bDebug == true) && (m_bFile == true))
	{
		if (m_bFileCreated == false)
		{
			if (m_bAppend == false)
			{
				clearFile();
			}

			m_bFileCreated = true;
		}

		logInFile(pMessage);

		logInDebugger(pMessage);
	}

	// Logging nur in Debugger
	else if ((m_bDebug == true) && (m_bFile == false))
	{
		logInDebugger(pMessage);
	}
	
	// Logging nur in File
	else if ((m_bDebug == false) && (m_bFile == true))
	{
		if (m_bFileCreated == false)
		{
			if (m_bAppend == false)
			{
				clearFile();
			}

			m_bFileCreated = true;
		}
		
		logInFile(pMessage);
	}
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::log
///////////////////////////////////////////////////////////////////////////////////////
// Info:			leitet int in den spezifizierten Context
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::log(int nMessage, bool bTimeStamp)
{
	char cMessage[16];
	char* pCheck = NULL;

	pCheck = _itoa(nMessage, cMessage, 10);

	if (pCheck == NULL)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::log > (ERROR!!!) could not convert int -> no logging performed\n");
		#endif

		return;
	}

	log(cMessage, bTimeStamp);
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::log
///////////////////////////////////////////////////////////////////////////////////////
// Info:			leitet double in den spezifizierten Context
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::log(double dMessage, bool bTimeStamp)
{
	char cMessage[32];

	_gcvt(dMessage, 15, cMessage);

	log(cMessage, bTimeStamp);
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::log
///////////////////////////////////////////////////////////////////////////////////////
// Info:			leitet long in den spezifizierten Context
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::log(long lMessage, bool bTimeStamp)
{
	char cMessage[16];
	char* pCheck = NULL;

	pCheck = _ltoa(lMessage, cMessage, 10);

	if (pCheck == NULL)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::log > (ERROR!!!) could not convert long -> no logging performed\n");
		#endif

		return;
	}

	log(cMessage, bTimeStamp);
}


///////////////////////////////////////////////////////////////////////////////////////
// Methode:			XLogging::log
///////////////////////////////////////////////////////////////////////////////////////
// Info:			leitet unsigned long in den spezifizierten Context
///////////////////////////////////////////////////////////////////////////////////////

void XLogging::log(unsigned long ulMessage, bool bTimeStamp)
{
	char cMessage[16];
	char* pCheck = NULL;

	//pCheck = _ultoa(ulMessage, cMessage, 10);

	if (pCheck == NULL)
	{
		#if defined (XDEBUG) || defined (XLOGGINGDEBUG)
		OutputDebugString("XLogging::log > (ERROR!!!) could not convert unsigned long -> no logging performed\n");
		#endif

		return;
	}

	log(cMessage, bTimeStamp);
}


/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	XLogging.cpp
/////////////////////////////////////////////////////////////////////////////////////
