/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		editor.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	23.07.2001
// letzte nderung:	06.11.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Hauptdatei des Editors		
//
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "editor.h"

// globale Variablen
int				iPosX=0;
int				iPosY=0;
int				iStatus=0;
bool			bShowStarts=true;
bool			bShowHelp=false;

char			pcFileName[1024]="level\\";

XSurface		xsurfField;
XSurface		xsBackLeft;
XSurface		xsBackRight;
XSprite			xsTileSet;
XSprite			xsSelect;
XFont			xfChars;

Field			fieldData;

EditorData		edGUI;

// Funktions-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::wndProc
/////////////////////////////////////////////////////////////////////////////////////
// Info:			DIE Window Procedure. Sorgt fuer die Verarbeitung von Windows-
//					Botschaften unabhaengig vom jeweiligen Programmstatus. Ruft am
//					Ende die Window Procedure des aktuellen Programteils auf.
/////////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK wndProc(HWND hWnd,
						 UINT message,
						 WPARAM wParam,
						 LPARAM lParam)
{
	switch (message)
	{
		case WM_KEYDOWN:
			if (wParam>='0' && wParam<='9')
				edGUI.execute(&fieldData,iPosX,iPosY,iStatus,(int)(wParam-'0'));
			switch (wParam)
			{
				case VK_ESCAPE: 
					PostMessage(hWnd,WM_CLOSE,0,0);
					break;
				case VK_UP:
					iPosY--;
					if (iPosY==-1)
						iPosY+=iFieldSizeY;
					break;
				case VK_DOWN:
					iPosY++;
					if (iPosY==iFieldSizeY)
						iPosY-=iFieldSizeY;
					break;
				case VK_LEFT:
					iPosX--;
					if (iPosX==-1)
						iPosX+=iFieldSizeX;
					break;
				case VK_RIGHT:
					iPosX++;
					if (iPosX==iFieldSizeX)
						iPosX-=iFieldSizeX;
					break;
				case VK_F1:
					iStatus=0;
					break;
				case VK_F2:
					iStatus=1;
					break;
				case VK_F3:
					iStatus=3;
					break;
				case VK_F4:
					iStatus=4;
					break;
				case VK_F5:
					iStatus=2;
					break;
				case VK_F6:
					iStatus=7;
					break;
				case VK_F7:
					iStatus=8;
					break;
				case VK_F8:
					iStatus=9;
					break;

				case 'Q':
					fieldData.setTile(iPosY,
									  iFieldSizeX-iPosX-1,
									  fieldData.getTile(iPosX,iPosY),
									  true);
					break;
				case 'W':
					fieldData.setTile(iFieldSizeY-iPosY-1,
									  iPosX,
									  fieldData.getTile(iPosX,iPosY),
									  true);
					break;
				case 'E':
					fieldData.setTile(iFieldSizeX-iPosX-1,
									  iFieldSizeY-iPosY-1,
									  fieldData.getTile(iPosX,iPosY),
									  true);
					break;
				case 'R':
					fieldData.setTile(iPosX,
									  iFieldSizeY-iPosY-1,
									  fieldData.getTile(iPosX,iPosY),
									  true);
					break;
				case 'T':
					fieldData.setTile(iFieldSizeX-iPosX-1,
									  iPosY,
									  fieldData.getTile(iPosX,iPosY),
									  true);
					break;

				case 'K':
					bShowStarts=!bShowStarts;
					break;
				case 'H':
					bShowHelp=!bShowHelp;
					break;
				case 'S':
					fieldData.save(pcFileName);
					break;
				case 'L':
					fieldData.load(pcFileName);
					break;
			}
			break;
		case WM_MOUSEMOVE:
			break;
		case WM_LBUTTONDOWN:
			break;
		case WM_LBUTTONUP:
			break;
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			// Bei Fokus-Verlust die Ausfuehrung der Anwendung anhalten
			if ((wParam==SIZE_MAXHIDE) || (wParam==SIZE_MINIMIZED))
			{
				g_bIsActive	= false;
			}
			else // sonst Ausfuehrung fortsetzen
			{
				g_bIsActive	= true;
			}
			break;
	} // switch

	// Window Procedure des aktuellen Programteils aufrufen
	g_pPPartActual->wndProc(hWnd,message,wParam,lParam);

	return ::DefWindowProc(hWnd,message,wParam,lParam);
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Ist fuer die Initialisierung der gesamten Anwendung verantwort-
//					lich.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT initialize(HINSTANCE	hInstance, char* pcCommandLine)
{
	HRESULT	hRes	= DD_OK;
	
	// Anwendungsfenster erzeugen
	g_xaApplication.initialize(hInstance,"Snake-Editor","Snake-Editor",500);

	// DirectDraw initialisieren
	hRes	=g_xdDisplay.initialize(1024,768,32,g_xaApplication.getHWnd());
	
	//Timer initialisieren
	g_xtTimer.setUp();

	//DInput initilaisieren
	g_xiInput.initialize(hInstance);
	g_xiInput.activateKeyboard(g_xaApplication.getHWnd());

	//Field Surface mit 768x768 initialisieren und fllen
	xsurfField.initialize(&g_xdDisplay,768,768);

	// Grafiken laden
	hRes=xsBackLeft.initialize(&g_xdDisplay,"gfx\\ingameleft.bmp");

	hRes=xsBackRight.initialize(&g_xdDisplay,"gfx\\ingameright_eddi.bmp");

	hRes=xsTileSet.initialize(&g_xdDisplay,"gfx\\tileset.spr");
	xsTileSet.m_bVisible=true;

	hRes=xsSelect.initialize(&g_xdDisplay,"gfx\\select.spr");
	xsSelect.m_bVisible=true;

	hRes=xfChars.initialize(&g_xdDisplay,"gfx\\charset.fnt");

	//Commandozeile auswerten
	while (*pcCommandLine!=0 && *pcCommandLine !='/')
		pcCommandLine++;
	if(*pcCommandLine =='/')
		pcCommandLine++;
	if (*pcCommandLine!=0)
	{
		strcat(pcFileName,pcCommandLine);
	}
	else
	{
		strcat(pcFileName,"sn---00");
	}
	strcat(pcFileName,".lvl");

	
	//Feld laden und malen
	fieldData.load(pcFileName);
	fieldData.render(&xsurfField,&xsTileSet,0,0,16);

	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Ist fuer die Deinitialisierung der gesamten Anwendung bei
//					Programmende verantwortlich.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT uninitialize(void)
{
	HRESULT	hRes	= DD_OK;

	// Grafiken entladen
	xsurfField.uninitialize();
	xsBackLeft.uninitialize();
	xsBackRight.uninitialize();
	xsTileSet.uninitialize();
	xsSelect.uninitialize();
	xfChars.uninitialize();
	
	//DInput initilaisieren
	g_xiInput.uninitialize();

	// DirectDraw deinitialisieren
	g_xdDisplay.uninitialize();
	
	return hRes;
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::recoverFromFocusLost
/////////////////////////////////////////////////////////////////////////////////////
// Info:			holt verlorene Surfaces zurueck
/////////////////////////////////////////////////////////////////////////////////////
void recoverFromFocusLost(void)
{
	// Grafiken restoren
	xsurfField.restore();
	xsBackLeft.restore();
	xsBackRight.restore();
	xsTileSet.restore();
	xsSelect.restore();
	xfChars.restore();
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::showHelp
/////////////////////////////////////////////////////////////////////////////////////
// Info:			zeigt Hilfe an
/////////////////////////////////////////////////////////////////////////////////////
void showHelp(void)
{
	xfChars.textOut(&g_xdDisplay,110,30,"f tasten waehlen steinart");
	xfChars.textOut(&g_xdDisplay,110,50,"k:koepfe an aus");
	xfChars.textOut(&g_xdDisplay,110,70,"l:laden");
	xfChars.textOut(&g_xdDisplay,110,90,"s:speichern");
	xfChars.textOut(&g_xdDisplay,110,110,"q:kopie nach 90 grad");
	xfChars.textOut(&g_xdDisplay,110,130,"w:kopie nach 270 grad");
	xfChars.textOut(&g_xdDisplay,110,150,"e:kopie nach 180 grad");
	xfChars.textOut(&g_xdDisplay,110,170,"r:kopie vertikal spiegeln");
	xfChars.textOut(&g_xdDisplay,110,190,"t:kopie horizontal spiegeln");
}

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::run
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Programmablauf
/////////////////////////////////////////////////////////////////////////////////////

void run(int iTime, XDisplay* pxdisplayDest)
{
	int			iTmp;
	char		pcBuf[20];

	xsBackLeft.blit(pxdisplayDest,0,0);

	//renderGameField
	fieldData.render(&xsurfField,&xsTileSet,0,0,16,false);
	xsurfField.blit(pxdisplayDest,80,0);

	xsBackRight.blit(pxdisplayDest,848,0);

	//Kpfe fr Startpositionen setzen
	if (bShowStarts)
	{
		for(iTmp=0;iTmp<iNumberOfPlayers;iTmp++)
		{
			xsTileSet.m_lX=80+16*fieldData.getStartX(iTmp);
			xsTileSet.m_lY=16*fieldData.getStartY(iTmp);
			xsTileSet.m_lFrame=134+32*iTmp+(fieldData.getStartDirection(iTmp)+2)%4;
			xsTileSet.blit(pxdisplayDest);
		}
	}

	//Men auf der rechten Seite aufbauen
	for(iTmp=0;iTmp<10;iTmp++)
	{
		sprintf(pcBuf,"%d=",(iTmp+1)%10);

		xfChars.textOut(pxdisplayDest,944,150+32*iTmp,pcBuf);

		xsTileSet.m_lX=980;
		xsTileSet.m_lY=150+32*iTmp;
		xsTileSet.m_lFrame=edGUI.getIcon(iStatus,(iTmp+1)%10);
		if (xsTileSet.m_lFrame>=0)
			xsTileSet.blit(pxdisplayDest);
	}

	//Koordinatenangabe
	sprintf(pcBuf,"x: %02d",iPosX);
	xfChars.textOut(pxdisplayDest,944,680,pcBuf);
	sprintf(pcBuf,"y: %02d",iPosY);
	xfChars.textOut(pxdisplayDest,944,705,pcBuf);

	//die Selectbox malen
	xsSelect.m_lX=80+16*iPosX;
	xsSelect.m_lY=16*iPosY;
	xsSelect.m_lFrame=0;
	xsSelect.blit(pxdisplayDest);

	//Hilfe zeigen
	if (bShowHelp) showHelp();

}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::WinMain
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Einsprungspunkt des Programmes
/////////////////////////////////////////////////////////////////////////////////////

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG			msg;

	// Loggen in Debugger UND Datei, anhaengen aktiviert
	XLogging::setContext(true,true,false,"editor_error.log");
	XLogging::log("\n--- Programmstart ---\n\n");

	// Programm-Startup
	::initialize(hInstance, lpCmdLine);

	while (1)
	{
		if (::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE))
		{
			if (!::GetMessage(&msg,NULL,0,0))
			{ // WM_QUIT bekommen
				XLogging::log("\nProgrammende eingeleitet...\n");
				
				// aktuellen Status deinitialisieren
				g_pPPartActual->uninitialize();
				delete g_pPPartActual;

				// while-Schleife beenden
				break;
			}
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
		else
		{
			if (g_bIsActive==true)
			{ // wenn, die Anwendung den Fokus hat...
				// ...aktuellen Programm-Teil ausfuehren
				
				run(g_xtTimer.update(),&g_xdDisplay);
				HRESULT hResult	= g_xdDisplay.flip();
				
				if (hResult==DDERR_SURFACELOST)
				{
					g_xdDisplay.restore();
					recoverFromFocusLost();
				}
				
			}
		}
	}

	// Programm-Shutdown
	::uninitialize();

	XLogging::log("\n--- Programmende ---\n\n");

	return msg.wParam;
}
/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	editor.cpp
/////////////////////////////////////////////////////////////////////////////////////
