/////////////////////////////////////////////////////////////////////////////////////
// Dateiname:		main.cpp
/////////////////////////////////////////////////////////////////////////////////////
// Autor:			Christian Roesch
//
// Erstelldatum:	23.07.2001
// letzte nderung:	25.11.2001
/////////////////////////////////////////////////////////////////////////////////////
// Info:	Hauptdatei			
//
//			das wird ein kleiner Snake-Clone... irgendwann... mal sehen...
//
/////////////////////////////////////////////////////////////////////////////////////

//includes
#include "main.h"

// Funktions-Definitionen

/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::wndProc
/////////////////////////////////////////////////////////////////////////////////////
// Info:			DIE Window Procedure. Sorgt fuer die Verarbeitung von Windows-
//					Botschaften unabhaengig vom jeweiligen Programmstatus. Ruft am
//					Ende die Window Procedure des aktuellen Programteils auf.
/////////////////////////////////////////////////////////////////////////////////////
LRESULT CALLBACK wndProc(HWND hWnd,
						 UINT message,
						 WPARAM wParam,
						 LPARAM lParam)
{
	switch (message)
	{
		case WM_KEYDOWN:
			switch (wParam)
			{
// this was for debug purposes, disabled for release build
//				case VK_END: 
//					PostMessage(hWnd,WM_CLOSE,0,0);
//					break;
			}
			break;
		case WM_MOUSEMOVE:
			break;
		case WM_LBUTTONDOWN:
			break;
		case WM_LBUTTONUP:
			break;
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_SIZE:
			// Bei Fokus-Verlust die Ausfuehrung der Anwendung anhalten
			if ((wParam==SIZE_MAXHIDE) || (wParam==SIZE_MINIMIZED))
			{
				g_bIsActive	= false;
			}
			else // sonst Ausfuehrung fortsetzen
			{
				g_bIsActive	= true;
			}
			break;
	} // switch

	// Window Procedure des aktuellen Programteils aufrufen
	g_pPPartActual->wndProc(hWnd,message,wParam,lParam);

	return ::DefWindowProc(hWnd,message,wParam,lParam);
}


/////////////////////////////////////////////////////////////////////////////////////
// Methode:			::loadHighScores
/////////////////////////////////////////////////////////////////////////////////////
// Info:	ldt die HighScores, falls das File nicht existiert wird mit einer
//			DefaultListe gefllt
/////////////////////////////////////////////////////////////////////////////////////
void loadHighScores(void)
{
	int iList;

	if(!g_hsData.load("highscr.dat"))
	{
		//Dummy Listen erzeugen
		for(iList=0;iList<2+2*iNumberOfChallengeLevel;iList++)
		{
			g_hsData.setEntry(iList,"- empty -",1000);
			g_hsData.setEntry(iList,"- empty -",800);
			g_hsData.setEntry(iList,"- empty -",600);
			g_hsData.setEntry(iList,"- empty -",200);
			g_hsData.setEntry(iList,"- empty -",100);
		}
	}
}
/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::setProgramState
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Setzt einen neuen Programmstatus. Dazu wird der alte de-
//					initialisiert, dann der Zeiger g_pActualProgramPart auf das
//					neue ProgramPart-Objekt verbogen und fuer dieses initialize
//					aufgerufen.
/////////////////////////////////////////////////////////////////////////////////////

void setProgramState(State	stateNew)
{
	// aktuelle configuration speichern
	g_configData.save();

	// alten Zustand deinitialisieren
	g_pPPartActual->uninitialize();

	delete g_pPPartActual;

	// Zeiger auf neu geforderten Zustand verbiegen
	// wenn ein nicht existenter Zustand angefordert wird, gibt es eine Absturz ?!!!
	switch (stateNew)
	{
	case Intro:
		g_pPPartActual=new PPIntro;//auf Intro schalten
		break;

	case Menu:
		g_pPPartActual=new PPMenu;//auf Menu schalten
		break;

	case Game:
		g_pPPartActual=new PPGame;//Ingame starten
		break;

	case Outro:
		g_pPPartActual=new PPOutro;//Outro beginnen
		break;

	} // switch

	if (g_pPPartActual==NULL) //kein Speicher um ProgrammTeil anzulegen, ahhhhhhhhh!!!
	{
		XLogging::log("::setProgramState > (FEHLER!!!) Zuwenig Speicher !!!\n");
		exit(666);//verdammt
	}

	// neuen Zustand initialisieren
	g_pPPartActual->initialize(&g_xdDisplay);

	//Zeit die eben vergangen ist ignorieren und Zhler auf 0 setzen
	g_xtTimer.update();
	g_xtTimer.setTime(0);
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::initialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Ist fuer die Initialisierung der gesamten Anwendung verantwort-
//					lich.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT initialize(HINSTANCE	hInstance)
{
	HRESULT	hRes	= DD_OK;

	int iBitPerPixel;

	int iOutPut;
	int iDriver;
	int iChannels;
	int iFreq;

	//Hardware cfg-File ffnen
	XConfigFile xcfRead("hardware.cfg");

	//Hardware-cfgs lesen
	xcfRead.getAsInt("BitPerPixel",iBitPerPixel,16);

	xcfRead.getAsInt("Output",iOutPut,2);
	xcfRead.getAsInt("Driver",iDriver,0);
	xcfRead.getAsInt("Channels",iChannels,32);
	xcfRead.getAsInt("MixFreq",iFreq,44100);
	
	// Anwendungsfenster erzeugen
	g_xaApplication.initialize(hInstance,"Snake","Snake",500);

	// DirectDraw initialisieren
	hRes	=g_xdDisplay.initialize(1024,768,iBitPerPixel,g_xaApplication.getHWnd());
	
	//ist gewollter Videomode in Ordnung?
	if (hRes==DD_OK)
	{   //alles wunderbar
		XLogging::log("=> desired Videomode initialized...\n");
	}
	else
	{   //funzt nicht, 16Bit probieren
		g_xdDisplay.uninitialize();
		XLogging::log("\n===> desired Videomode failed...\n\n");
		XLogging::log("=> trying 1024x768x16\n");
		hRes=g_xdDisplay.initialize(1024,768,16,g_xaApplication.getHWnd(),DDSCL_EXCLUSIVE|DDSCL_FULLSCREEN,false);
		
		//hat der gefunzt?
		if (hRes==DD_OK)
		{   // jawohl
			XLogging::log("=> 1024x768x16 initialized...\n");
		}
		else
		{   //was isn das frn Scheissteil ???
			XLogging::log("=> 1024x768x16 Videomode failed...\n");
			XLogging::log("\n===> Error!!! This videocard is not supported\n\n");
			MessageBox(	g_xaApplication.getHWnd(),
						"DDraw Init failed! [VideoMode is not supported]",
						"Exiting now...",
						MB_ICONERROR | MB_OK );
		}

	}

	
	//Timer initialisieren
	g_xtTimer.setUp();

	//DInput initilaisieren
	g_xiInput.initialize(hInstance);
	g_xiInput.activateKeyboard(g_xaApplication.getHWnd());

	//HardWare SoundMixing nicht nutzen wenn die Karte
	//weniger als 8 Kanle kann
	FSOUND_SetMinHardwareChannels(8);

	//Soundengine initialisieren
	if (!g_xfmodHelper.initialize(	iOutPut,
									iDriver,
									iFreq,
									iChannels))
	{
		//bei Fehler nochmal mit Standardwerten
		g_xfmodHelper.uninitialize();
		g_xfmodHelper.initialize();
	}
	
	//SFX und Music-Volume setzen
	g_xfmodHelper.m_iSFXVolume=11*g_configData.m_iSFXVolume;
	g_xfmodHelper.m_iMusicVolume=11*g_configData.m_iMusicVolume;

	//Kanle fr SFX reservieren

	
	//HighScore-Daten laden
	loadHighScores();

	// starting intro
	::setProgramState(Intro);

	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::uninitialize
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Ist fuer die Deinitialisierung der gesamten Anwendung bei
//					Programmende verantwortlich.
/////////////////////////////////////////////////////////////////////////////////////

HRESULT uninitialize(void)
{
	HRESULT	hRes	= DD_OK;


	//SoundEngine beenden
	g_xfmodHelper.uninitialize();
	
	//DInput initilaisieren
	g_xiInput.uninitialize();

	// DirectDraw deinitialisieren
	g_xdDisplay.uninitialize();

	
	
	return hRes;
}


/////////////////////////////////////////////////////////////////////////////////////
// Funktion:		::WinMain
/////////////////////////////////////////////////////////////////////////////////////
// Info:			Einsprungspunkt des Programmes
/////////////////////////////////////////////////////////////////////////////////////

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG			msg;
	State		stateNext;
	HRESULT		hRes;

	// Loggen in Debugger UND Datei, anhaengen aktiviert
	XLogging::setContext(true,true,false,"error.log");
	XLogging::log("\n--- Programmstart ---\n\n");


	// Programm-Startup
	srand( (unsigned)time( NULL ) );

	hRes=::initialize(hInstance);

	//initialize failed...
	if (hRes!=DD_OK)
	{
		PostMessage(g_xaApplication.getHWnd(),WM_CLOSE,0,0);	
	}



	while (1)
	{
		if (::PeekMessage(&msg,NULL,0,0,PM_NOREMOVE))
		{
			if (!::GetMessage(&msg,NULL,0,0))
			{ 
				XLogging::log("\nProgrammende eingeleitet...\n");
				
				// aktuellen Status deinitialisieren
				g_pPPartActual->uninitialize();
				delete g_pPPartActual;

				// while-Schleife beenden
				break;
			}
			::TranslateMessage(&msg);
			::DispatchMessage(&msg);
		}
		else
		{
			if (g_bIsActive==true)
			{ // wenn, die Anwendung den Fokus hat...
				// ...aktuellen Programm-Teil ausfuehren
				stateNext=g_pPPartActual->run(g_xtTimer.update(),&g_xdDisplay);
				if (stateNext==WantToQuit)
				{
					PostMessage(g_xaApplication.getHWnd(),WM_CLOSE,0,0);
				}
				else if (stateNext!=g_pPPartActual->getState())
				{
					setProgramState(stateNext);
				}
				HRESULT hResult	= g_xdDisplay.flip();
				
				if (hResult==DDERR_SURFACELOST)
				{
					g_xdDisplay.restore();
					g_pPPartActual->recoverFromFocusLost();
				}
				
			}
		}
	}

	// Programm-Shutdown
	::uninitialize();

	XLogging::log("\n--- Programmende ---\n\n");

	return msg.wParam;
}
/////////////////////////////////////////////////////////////////////////////////////
// Ende der Datei:	main.cpp
/////////////////////////////////////////////////////////////////////////////////////
